        /* * * * * * * * * * * * * * * * *    
	 *  MARCconv        Version 2.0  *
         *   File:        datastruct.h   *
         *   data structure for tables   *
	 *     and record components     *
         *    Last update: 30/12/1996    *    
         *      Michael S. Preminger     *    
         *      BRODD, OSLO COLLEGE      *    
         *        THE ONE PROJECT        *    
         * * * * * * * * * * * * * * * * */   
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <time.h>
#include "regex.h"
#include <setjmp.h>
/*#include <malloc_dbg.h>*/
#ifndef MARC_DATASTRUCT

#define MAX_ITEM_LENGTH_IN_TABLE_FILE 300
#define MAX_TABLES 30
#define MARC_DATASTRUCT
#define LONGEST_CONSULT 30
#define LONGEST_SUB_PORTION 200
#define LONGEST_INDICATOR 30
#define LONGEST_VAR_DATA_PORTION 30
#define LONGEST_DECISION_TABLE_LOW_LEVEL 200
#define LONGEST_DECISION_TABLE_HIGH_LEVEL 50
#define CNT_POSSIBLE_INDICATOR_VALUES 11
#define CNT_POSSIBLE_INDICATOR_VALUES_PLUS_1 12
#define LEADER_LENGTH 24
#define LONGEST_LENGTH_LENGTH 4
#define LONGEST_START_POSITION_LENGTH 5
#define END_OF_FIELD '\x1E'
#define END_OF_RECORD '\x1D'
#define START_OF_SUBFIELD '\x1F'
#define MAX_SUBFIELDS_IN_DATA_FIELD 50
#define MAX_FIELDS_IN_RECORD 200
#define DIRECTORY_ENTRY_LENGTH 12 /*3+4+5*/
#define LONGEST_DATA_CODED_PORTION 30
#define LONGEST_SUB_PORTION_TABLE_ISBD 10
#define LONGEST_SUB_PORTION_CONDITIONAL 25
#define LONGEST_PORTION_TABLE_ISBD 20
#define PRIVATE_FIELD_COMMENT_SIZE 300
#define PRIVATE_FIELD_FORMAT_NAME_SF 'A'
#define PRIVATE_FIELD_CONTENT_SF 'B'
#define PRIVATE_FIELD_COMMENT_SF 'C'
#define PRIVATE_FIELD_CITE_SF_A_COMMENT_SF 'D'

#define MAX_RECORD_LENGTH 8000
#define RCD_SUTRS_LENGTH 18000
#define SUTRS_ARRAY_LENGTH 500
#define LONGEST_PREFIX_PORTION 50
typedef struct InitialIndicatorTable{
  int ArrayLength;
  char tag[50][4];
  /*We wish to allow referencing indicator1
   and indicator2 in their "natural" names:
   indicator[1] and indicator[2]*/
  char indicators[50][4];

}InitialIndicatorTable;
#undef TRUE
#undef FALSE
typedef enum Boolean{TRUE=0, FALSE=1} Boolean;

/* * * * * * * * * * * * * * * * *
 *                               *
 * CONVERSION TABLE STRUCTURES   *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct subPortionTable_Control{
  /*WhatType whatType=controlField;*/
  int Arraylength;
  int fromPosition[LONGEST_SUB_PORTION];
  char fromSubField[LONGEST_SUB_PORTION];
  int fromLength[LONGEST_SUB_PORTION];
  char *fromValue[LONGEST_SUB_PORTION];
  char *toTag[LONGEST_SUB_PORTION];
  char toSubField[LONGEST_SUB_PORTION];
  int toPosition[LONGEST_SUB_PORTION];
  int toLength[LONGEST_SUB_PORTION];
  char *toValue[LONGEST_SUB_PORTION];
} subPortionTable_Control;

typedef struct PortionTable_Control{
  int Arraylength;
  char *consultValue1[LONGEST_CONSULT];
  char *consultValue2[LONGEST_CONSULT];
  subPortionTable_Control *subPortion_CTL[LONGEST_CONSULT];

}PortionTable_Control;


typedef struct PortionTable_VarDat{
  int ArrayLength;
  /*PARALLEL ARRAYS FOR INDICATOR 1*/


  int  ind1ArrayLength;

  /*             terminate by null, allow search by
		 standard character functions */
  char ind1fromValue[CNT_POSSIBLE_INDICATOR_VALUES_PLUS_1];
  char ind1function[CNT_POSSIBLE_INDICATOR_VALUES];
  int /*either 1 or 2*/ ind1toInd[CNT_POSSIBLE_INDICATOR_VALUES];
  char ind1toValue[CNT_POSSIBLE_INDICATOR_VALUES];
  char *ind1special[CNT_POSSIBLE_INDICATOR_VALUES];/*to_function*/

  /*PARALLEL ARRAYS FOR INDICATOR 2*/


  int  ind2ArrayLength;

  /*             terminate by null, allow search by
		 standard character functions */
  char ind2fromValue[CNT_POSSIBLE_INDICATOR_VALUES_PLUS_1];
  char ind2function[CNT_POSSIBLE_INDICATOR_VALUES];
  int /* either 1 or 2*/ind2toInd[CNT_POSSIBLE_INDICATOR_VALUES];
  char ind2toValue[CNT_POSSIBLE_INDICATOR_VALUES];
  char *ind2special[CNT_POSSIBLE_INDICATOR_VALUES];/*to_function*/


  /*PARALLEL ARRAYS FOR SUB FIELD CODES*/
  char fromSubfieldCode[LONGEST_VAR_DATA_PORTION];
  char fromFunction[LONGEST_VAR_DATA_PORTION];
  char *treatment_par[LONGEST_VAR_DATA_PORTION]; /*to_FUNCTTION*/
  char *toSubField[LONGEST_VAR_DATA_PORTION];


  /*TO TAG*/
  /*Since  every portion table in practice represents a single
   toTag value, only a scalar is necessary*/
  char *toTag;
  char *text;

  /*MORE GROUPS OF INDICATORS/SUBFIELDS FOR A SINGLE TAG */
  struct PortionTable_VarDat *NextPT_DT;

}PortionTable_VarDat;


typedef struct generic_table{
  PortionTable_Control *portion_table_ctl;
  PortionTable_VarDat  *portion_table_vdt;
  Boolean specialTreatment;
  Boolean prefixTreatment;
  Boolean xiferpTreatment;
  Boolean copyField;
  char *defaultText;
  char *copyTag;
}generic_table;

typedef struct conversion_tbl_str{
  generic_table *Conversion_Table[1000];
}conversion_tbl_str;

/* * * * * * * * * * * * * * * * *
 *                               *
 *   DECISION TABLE STRUCTURES   *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct PortionTable_Decision{
  int ArrayLength;
  char indicator1;
  char indicator2;
  char *material;
  char subFieldCode;
  char *toTag; /**NOT NECESSARY. OUT.**/
  char *defaultToTag;

  /**INSTEAD OF PLACING TEXT STRINGS AND RELATED TO_TAGS
   IN ADJACENT ARRAYS, YOU COULD PLACE THEM EACH IN A
   LONG TEXT STRING, DELIMITED BY SPACE. DECISION
   IS SO MADE BY COUNTING
   UP WHILE SEARCHING FOR THE TEXT STRING, AND
   THEN COUNTING UP TO THE PROPER TO_TAG.

   ALTERNATIVELY, A SINGLE STRING COULD BE USED, LISTING
   EACH TO_TAG AFTER ITS TEXT STRING, and so on.**/
  char *lowerLevelTable_txt[LONGEST_DECISION_TABLE_LOW_LEVEL];
  char *lowerLevelTable_toTag[LONGEST_DECISION_TABLE_LOW_LEVEL];
  struct PortionTable_Decision *NextPT_DCS;
}PortionTable_Decision;

/*somewhat "heavy" definition for a structure.
eases parameter transfer in function calls*/
typedef struct decision_table_str{
  int ArrayLength;
  char *fromTags[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  PortionTable_Decision *Table[LONGEST_DECISION_TABLE_HIGH_LEVEL];
}decision_table_str;


/* * * * * * * * * * * * * * * * *
 *                               *
 *  INDICATOR TABLE STRUCTURES   *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct PortionTable_Indicator{
  int ArrayLength;
  /*int whichInd;*/

  char *checkField;/*
		    * If a foreign field is to be checked.
		    *
		    * When NULL check the from field 
		    * in the indicator_table_str pointing
		    * to this PortionTable_Indicator
		    */

  regex_t *checkField_compExp;
  char checkSubField;
  
  char condition;
  char *parameter;

  /*
   * The lowerLevelTable is a group of parallel arrays
   * pointed to from the indicator table, when more than
   * a single expression is to be evaluated within a
   * single conditional situation. 
   * See the printed documentation  
   */
  char *lowerLevelTable_txt[LONGEST_DECISION_TABLE_LOW_LEVEL];
  regex_t *lowerLevelTable_compExp[LONGEST_DECISION_TABLE_LOW_LEVEL];
  char lowerLevelTable_toInd[LONGEST_DECISION_TABLE_LOW_LEVEL];


  char nonDefaultToInd;
  char defaultToInd;
  struct PortionTable_Indicator *NextPT_IND;
}PortionTable_Indicator;

/*somewhat "heavy" definition for a structure.
eases parameter transfer in function calls*/
typedef struct indicator_table_str{
  int ArrayLength;
  char *fromTags[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  char *toTags[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  regex_t *toTags_compExp[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  /*May be 1 or 2 (could be enum)*/
  int whichInds[LONGEST_DECISION_TABLE_HIGH_LEVEL];

  PortionTable_Indicator *Table[LONGEST_DECISION_TABLE_HIGH_LEVEL];
}indicator_table_str;

/* * * * * * * * * * * * * * * * *
 *                               *
 *  CONDITIONAL CONVERSION TABLE *
 *         STRUCTURES            *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct PortionTable_Conditional{
  int ArrayLength;
  char fromSubFieldCode[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  char condition[LONGEST_VAR_DATA_PORTION];

  /*
   * Corresponds to the FROM (FUNCTION) column 
   * in the vd conversion table
   */
  char fromFunction[LONGEST_VAR_DATA_PORTION];

  char *sf_contents[LONGEST_VAR_DATA_PORTION];
  regex_t *sf_content_compExp[LONGEST_VAR_DATA_PORTION];
  char *parameter[LONGEST_VAR_DATA_PORTION];
  char *toSubField[LONGEST_VAR_DATA_PORTION];/*Corresponds to the S2 column in the conversion table*/
  char *treatment_par[LONGEST_VAR_DATA_PORTION];/*Corresponds to the TO column in the conversion table*/

}PortionTable_Conditional;

/*somewhat "heavy" definition for a structure.
eases parameter transfer in function calls*/
typedef struct conditional_table_str{
  int ArrayLength;
  char *fromTags[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  char *toTags[LONGEST_DECISION_TABLE_HIGH_LEVEL];
  PortionTable_Conditional *Table[LONGEST_DECISION_TABLE_HIGH_LEVEL];
}conditional_table_str;

/* * * * * * * * * * * * * * * * *
 *                               *
 *       PREFIXING TABLE         *
 *         STRUCTURES            *
 *  FOR UNIMARC LINKING FIELDS   *
 *                               *
 * * * * * * * * * * * * * * * * */
/*
 *  1. PREFIX
 */
typedef struct PortionTable_Prefix{
  int ArrayLength;
  char fromSubFieldCode[LONGEST_PREFIX_PORTION];

  /*
   * Corresponds to the FROM (FUNCTION) column 
   * in the vd conversion table
   */
  char fromFunction[LONGEST_PREFIX_PORTION];

  char *sf_contents[LONGEST_PREFIX_PORTION];
#ifdef SPLIT_REGEX
  struct re_pattern_buffer *sf_content_compExp[LONGEST_PREFIX_PORTION];
#else /*SPLIT_REGEX*/
  regex_t *sf_content_compExp[LONGEST_PREFIX_PORTION];
#endif/*SPLIT_REGEX*/
  char *parameter[LONGEST_PREFIX_PORTION];
  char *toField[LONGEST_PREFIX_PORTION];/*Corresponds to the S2 column in the conversion table*/
  char *treatment_par[LONGEST_PREFIX_PORTION];/*Corresponds to the TO column in the conversion table*/

}PortionTable_Prefix;

/*somewhat "heavy" definition for a structure.
eases parameter transfer in function calls*/
typedef struct prefixing_table_str{
  int ArrayLength;
  char *fromTags[LONGEST_PREFIX_PORTION];
  char fromInd1[LONGEST_PREFIX_PORTION];
  char fromInd2[LONGEST_PREFIX_PORTION];
  PortionTable_Prefix *Table[LONGEST_PREFIX_PORTION];
}prefixing_table_str;
/*
 *  2. XIFERP - The opposite of Prefix, When converting 
 *     FROM UNIMARC
 */

typedef struct PortionTable_Xiferp{
  int ArrayLength;

  char fromSubFieldCode[LONGEST_PREFIX_PORTION];/*Corresponds to the S1 column in the conversion table*/

  char fromFunction[LONGEST_PREFIX_PORTION];/*Corresponds to the FROM column in the conversion table*/

  char *toSFCodes[LONGEST_PREFIX_PORTION];/*Corresponds to the S2 column in the conversion table*/
  char *treatment_par[LONGEST_PREFIX_PORTION];/*Corresponds to the TO column in the conversion table*/

}PortionTable_Xiferp;

/*somewhat "heavy" definition for a structure.
eases parameter transfer in function calls*/
typedef struct xiferping_table_str{
  int ArrayLength;
  char *fromTags[LONGEST_PREFIX_PORTION];
  char *ToTags_text[LONGEST_PREFIX_PORTION];
  regex_t *ToTags_compExp[LONGEST_PREFIX_PORTION];
  PortionTable_Xiferp *Table[LONGEST_PREFIX_PORTION];
}xiferping_table_str;



/* * * * * * * * * * * * * * * * *
 *                               *
 *   ISBD PUNCTUATION TABLES     *
 *      FOR USMARC OUTPUT        *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct subPortionTable_ISBD{
  int ArrayLength;
  char subseq_sfc[LONGEST_SUB_PORTION_TABLE_ISBD];
  char function[LONGEST_SUB_PORTION_TABLE_ISBD];
  char *treatment_par[LONGEST_SUB_PORTION_TABLE_ISBD];
  char *Punctuation[LONGEST_SUB_PORTION_TABLE_ISBD];
}subPortionTable_ISBD;

typedef struct PortionTable_ISBD{
  int ArrayLength;
  char subFieldCodes[LONGEST_PORTION_TABLE_ISBD];
  subPortionTable_ISBD *spt_ISBD[LONGEST_PORTION_TABLE_ISBD];
}PortionTable_ISBD;

typedef struct isbd_table_str{
  PortionTable_ISBD *isbd_tbl[1000];
}isbd_table_str;


/* * * * * * * * * * * * * * * * *
 *                               *
 *        LOOKUP TABLES          *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct lookup_table{
  /*A lookup table structure for a certain format*/
  char *fromFormatName;
  char *toFormatName;
  conversion_tbl_str *cns;
  decision_table_str *dts; 
  indicator_table_str *its;
  conditional_table_str *cts;
  prefixing_table_str  *pts;
  xiferping_table_str *xts;
}lookup_table;


/***************************************************************
  END OF LOOKUP-TABLE RELATED STUFF
***************************************************************/

/* * * * * * * * * * * * * * * * *
 *                               *
 *      ISO2709 STRUCTURES       *
 *                               *
 * * * * * * * * * * * * * * * * */

typedef struct subField{
  int index; /*position in field*/
  int length;
  char subFieldCode;
  char *data;
}subField;

typedef struct dataField{
  int ArrayLength;
  char tag[4];
  char *text;
  char indicators[3];		/*So we can call indicator 2
				  for indicators[2] rather than
				  indicators[1].
				  indicators[0] remains unused*/

  char subFieldCodes[MAX_SUBFIELDS_IN_DATA_FIELD];
  subField *subFields[MAX_SUBFIELDS_IN_DATA_FIELD];
  /*dataField *nextRepeated;*/
}dataField;

typedef struct controlField{
  char tag[4];
  char *data;
}controlField;

typedef struct field{		/*GENERIC FIELD*/
  /*It is tempting to put the tag in here,
    but since we sometimes need to work with a
    data/control field type, it may be wise
    to be able to access their tag without
    going back to the generic pointer.*/

  dataField *datafield;
  controlField *ctrlfield;
}field;

typedef struct record{

  char format[20]; 
  char *string2709; /*Only used for input record.*/
  int ArrayLength;
  /**1.Here comes LABEL and DIRECTORY?**/


  /*We enjoy the fact that the in-record arrives SORTED,
    Directory can be deleted and "freed" after organizing
   and organize it in easily accessible arrays*/
  char leader[LEADER_LENGTH+1];
  char tags[MAX_FIELDS_IN_RECORD][4]; /*parallel tag array*/
  int lgt[MAX_FIELDS_IN_RECORD];/*dataLength*/
  int pos[MAX_FIELDS_IN_RECORD];
  int dataLength;
  field *fields[MAX_FIELDS_IN_RECORD]; /*parallel field-pointer Array*/
}record;



/*Boolean special_treatment; **A temporary solution to the problem of
			    extracting this Boolean without passing
			    on and on to many functions!**/
/*A FILE DESCRIPTOR FOR THE TOOLBOX ERROR LOGGER*/

int fildes ;
Boolean readTexts;
int lt_ArrayLength;
lookup_table *lt[MAX_TABLES];
isbd_table_str *iss;
#ifdef MARC_TEST_VARIABLES
dataField *globdtFld;
int globInt1;
#endif /*MARC_TEST_VARIABLES*/
record *globRecord;
#ifndef _ONETK_LIB_BUILD_
int rcd_cnt;

#ifdef MARC_MULTI_TABLE
/* * * * * * * * * * * * * * * * *
 *                               *
 *       GLOBAL VARIABLES        *
 *                               *
 * * * * * * * * * * * * * * * * */

Boolean readTexts;

#endif
#endif
#define MARC_DATASTRUCT
#endif
 









