        /* * * * * * * * * * * * * * * * *    
	 *  MARCconv        Version 2.3  *
         *   File:       outputRecord.c  *    
	 *        Output treatment       * 
	 *          of records           *
         *    Last update: 28/01/1997    *    
         *      Michael S. Preminger     *    
         *      BRODD, OSLO COLLEGE      *    
         *        THE ONE PROJECT        *    
         * * * * * * * * * * * * * * * * */

/* * * * * * * * * * * * * * * * * * * * 
 *  This file includes routines that   *
 *  sort the fields, add ISBD          *
 *  punctuation and visualize records  *
 * * * * * * * * * * * * * * * * * * * */

/* * * * * * * * * * * * * * * * * * * * 
 *  TODO list                          *
 *  program a function that            *
 *  punctuate subfield x if subfield y *
 *  is present in the field (regardless*
 *  of position relative to x          *
 *-------------------------------------*
 *  program a function that            *
 *  punctuate subfield x if subfield y *
 *  follows x in the field             *
 *  (immediately or otherwise)          *
 *                                     *
 * * * * * * * * * * * * * * * * * * * */


   
#include <setjmp.h>

#include "MARCproto.h"



extern jmp_buf marcconv_env;

/* * * * * * * * * * * * * * * * *
 *                               *
 *      UTILITY FUNCTIONS        *
 *                               *
 * * * * * * * * * * * * * * * * */

char *itoalgt(int n,  
	      int lgt)
{
  char *ret;
  ret=calloc((1+lgt),sizeof(char));
  sprintf( ret, "%0*.*d", lgt, lgt, n );
  return ret;		
}

/* * * * * * * * * * * * * * * * *
 *                               *
 *        PREPARING FOR          *
 *           OUTPUT              *
 *                               *
 * * * * * * * * * * * * * * * * */

/*%%%%% SORTING Of FIELDS %%%%%%%*/

void swapFields(record *rcd, 
		int current, 
		int smallest)
{
  char *tag=(char *)calloc(4,sizeof(char));
  field *fld;
  int lgt;
  
  fld=rcd->fields[current];
  strncpy(tag,rcd->tags[current],4);
  /*lgt=rcd->lgt[current];*/

  strncpy(rcd->tags[current],rcd->tags[smallest],4);
  rcd->fields[current]=rcd->fields[smallest];
  /*rcd->lgt[current]=rcd->lgt[smallest];*/
  
  strncpy(rcd->tags[smallest],tag,4);
  rcd->fields[smallest]=fld;
  /*rcd->lgt[current]=rcd->lgt[smallest];*/
  
  free(tag);
}

/** to do
 * The sorting is done simply. This may not always be
 * sufficient, because sometimes fields will be required to
 * come in clusters. An example will be associated holding fields
 * as planned for the canadian union catalogue:
 * http://www.nic-bnc.ca/resource/vcuc/ehold.htm
 **/
void sortFields(record *rcd){

  int n, k,l, small;
  n=rcd->ArrayLength;
  if (rcd->ArrayLength>1){
    for (k=0; k < n-1; k++) {
      small=k;
      for (l=k+1;l<n;l++){
	if (atoi(rcd->tags[l]) < atoi(rcd->tags[small]))
	  small=l;

      }
      if (small!=k)
	swapFields(rcd,k,small);      
    }
  }

}

/*%%%%% UPDATING CORRECT LENGTHS %%%%%%%*/
/*%%%%%%%%OF SUBFIELDS/FIELDS %%%%%%%%%%*/

int updateRecordFieldLengths(record *rcd){
  int i,j;
  dataField *dtFld;
  rcd->dataLength=0;
  /*starting with i=1 because [0] is the leader, and
   is addressed elsewhere in the code*/
  for (i=1;i<rcd->ArrayLength;i++){

    if (rcd->fields[i]->ctrlfield!=NULL){
		/*NSU, and correct everywhere. This is hard coding of information!!!*/
	  char **data=(&rcd->fields[i]->ctrlfield->data);
	  int lgt=strlen(*data);
      if (*data!=NULL){

		if (strcmp(rcd->tags[i], "008")==0 && lgt < 40){
			*data=(char *)realloc (*data, (40+1)*sizeof(char));
			memset((*data)+lgt,' ' , 40-lgt);
			memset((*data)+40, '\0',1);
			lgt=strlen(*data);
		}
			
		rcd->lgt[i]=
			1 + /*Include end_of_field*/
				strlen(rcd->fields[i]->ctrlfield->data);
	
	
      }
  

      else{
	rcd->lgt[i]=0;
      }
    }
    if (rcd->fields[i]->datafield!=NULL){
      dtFld=rcd->fields[i]->datafield;
      rcd->lgt[i]=
	1/*end_of_field*/
	+2;/*indicators*/

      /*Go through sufields*/
      for(j=0;
	  j<dtFld->ArrayLength;
	  j++) {
	rcd->lgt[i]+=2;/*subField prefix + code*/
	if (dtFld->subFields[j]->data!=NULL){
	
	  rcd->lgt[i]+=
		  strlen(
		       dtFld->subFields[j]->data);
	}
      }
    }
  }
}

int updateRecordDataLength(record *rcd){
  
  int i;
  for (i=1;i<rcd->ArrayLength;i++)
    rcd->dataLength=rcd->dataLength+rcd->lgt[i];
}

/*%%%%%%% UPDATING Indicator VALUES %%%%%%%%*/
int updateUndefinedIndicatorValues(record *rcd,
				   char undefined){
  int i,j;
  dataField *dtFld;
  for (i=1;i<rcd->ArrayLength;i++){
    if (rcd->fields[i]->datafield!=NULL){
      dtFld=rcd->fields[i]->datafield;
      if (dtFld->indicators[1]=='#')
	dtFld->indicators[1]=undefined;
      if (dtFld->indicators[2]=='#')
	dtFld->indicators[2]=undefined;
      
    }
  }
}


/*%%%%%%% ISBD Punctuation Functions %%%%%%%%*/

char *concat(char *str1,
	     char *str2)
{
  char *tmp;
  int length=strlen(str1)+strlen(str2)+3;
  tmp=(char *)malloc(length);
  memset(tmp,'\0',length);
  if (strlen(str1)>0)
    strncat(tmp, str1, strlen(str1));
  if (strlen(str2)>0)
    strncat(tmp, str2, strlen(str2));
  return tmp;
}



Boolean treatISBD(field *fld,
		  int sfIndex,
		  subPortionTable_ISBD *si,
		  int siIndex)
{
  int i; /*counter for the si array entries*/
  dataField *dtFld;
  char *tmp;
  subField *csf; /*current-sub-field*/
  subField *psf; /*previous sub field*/
  dtFld=fld->datafield;
  csf=dtFld->subFields[sfIndex];

  
  if (strcmp(si->Punctuation[siIndex],"(")==0)
    /*Punctuation leading the subfield data: '('*/
    switch(si->function[siIndex]){
    case '\0':
      tmp=csf->data;
      csf->data=concat("(", csf->data);
      free(tmp);
      return TRUE;
      break;
    case '!':/*None of parameter subfields precede*/
      if (strchr(si->treatment_par[siIndex],
		dtFld->subFieldCodes[sfIndex-1])==NULL){
	tmp=csf->data;
	csf->data=concat("(", csf->data);
        free(tmp);
     
      }
      return TRUE;
      break;
    case '+':
      {
	char m[200];
	sprintf(m, 
		"treatISBD()\n   a + function is never associated with an opening parenthes. ISBD table error.\n");
	if (ONETK_WriteLogFile(fildes, m)==-1){
	  /**Error handle**/
	}
      }
      break;
    }
    else
      /*Punctuation at end of sub field*/
      switch(si->function[siIndex]){
      case '\0':
	tmp=csf->data;
	csf->data=concat(csf->data, si->Punctuation[siIndex]);
	free(tmp);
	break;
      case '!':/*None of parameter subfield precede*/
	if (strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex-1])==NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	break;
      case '+':/*One of parameter subfield precedes*/
	if (strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex-1])!=NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	
	break;
      case '&': /*none of parameter subfields succeeds $*/
	if (sfIndex==dtFld->ArrayLength-1
	    ||
	    strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex+1])==NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	else return TRUE;
	break;
      }
      

    return FALSE;
}


Boolean treatISBD_herpet(field *fld,
		  int sfIndex,
		  subPortionTable_ISBD *si,
		  int siIndex)
{
  int i; /*counter for the si array entries*/
  dataField *dtFld;
  char *tmp;
  subField *csf; /*current-sub-field*/
  subField *psf; /*previous sub field*/
  dtFld=fld->datafield;
  csf=dtFld->subFields[sfIndex];

  
  if (strcmp(si->Punctuation[siIndex],"(")==0){
    /*Punctuation leading the subfield data: '('*/
    switch(si->function[siIndex]){
    case '\0':/*unconditional*/
      tmp=csf->data;
      csf->data=concat("(", csf->data);
      free(tmp);
      if (sfIndex==dtFld->ArrayLength-1){
	tmp=csf->data;
	csf->data=concat(csf->data, ")");
	free(tmp);
	/*	return FALSE;
      }else
	/*	return TRUE;*/
      }
      break;
    case '!':/*None of parameter subfields precede*/
      if (strchr(si->treatment_par[siIndex],
		 dtFld->subFieldCodes[sfIndex-1])==NULL){
	tmp=csf->data;
	csf->data=concat("(", csf->data);
	free(tmp);
	if (sfIndex==dtFld->ArrayLength-1){
	  tmp=csf->data;
	  csf->data=concat(csf->data, ")");
	  free(tmp);
	}
       
      }
      return TRUE;
      break;
    case '+':
      {
	char m[200];
	sprintf(m, 
		"treatISBD()\n   a + function is never associated with an opening parenthes. ISBD table error.\n");
	if (ONETK_WriteLogFile(fildes, m)==-1){
	  /**Error handle**/
	}
      }
      break;
    }
  }
  else
      /*Punctuation at end of sub field*/
      switch(si->function[siIndex]){
      case '\0':
	tmp=csf->data;
	csf->data=concat(csf->data, si->Punctuation[siIndex]);
	free(tmp);
	break;
      case '!':/*None of parameter subfield precede*/
	if (strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex-1])==NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	break;
      case '+':/*One of parameter subfield precedes*/
	if (strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex-1])!=NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	
	break;
      case '&': /*none of parameter subfields succeeds $*/
	if (sfIndex==dtFld->ArrayLength-1
	    ||
	    strchr(si->treatment_par[siIndex],
		   dtFld->subFieldCodes[sfIndex+1])==NULL){
	  tmp=csf->data;
	  csf->data=concat(csf->data, si->Punctuation[siIndex]);
	  free(tmp);
	}
	else return TRUE;
	break;
      }
      

    return FALSE;
}
  

subPortionTable_ISBD *findSubTableISBD(PortionTable_ISBD *pi, 
				       char sfcode)
{
  int i;
  for (i=0;i<pi->ArrayLength;i++){
    if (pi->subFieldCodes[i]==sfcode)
      return pi->spt_ISBD[i];
  }
  return NULL;
}
void *updateIsbdPunctuation(record *rcd,
			    PortionTable_ISBD *itbl[])
{
  int i; /*field (portion table) counter*/
  int j;/*sub field counter*/
  int k; /*sub portion table counter*/
  field *fld;
  dataField *dtFld;
  PortionTable_ISBD *pi;
  subPortionTable_ISBD *spi;
  Boolean Opp=FALSE;

  for (				/*All fields*/
       i=0;i<rcd->ArrayLength;i++){

    if (itbl[atoi(rcd->tags[i])]!=NULL){
      pi=itbl[atoi(rcd->tags[i])];
      fld=rcd->fields[i];
      dtFld=fld->datafield;

      for ( /*Actual subfields in field*/
	   j=0;j<dtFld->ArrayLength;j++){

	if (dtFld->subFieldCodes[j] == 'q'){
	  printf("stopp her");
	}
	spi=findSubTableISBD(pi, dtFld->subFieldCodes[j]);

	if (spi!=NULL){
	  for (	/*All occurences of sub field 
		  code in table sub portion*/
	       k=0;k< spi->ArrayLength;k++){
	    if (/*subseq=NULL*/
		spi->subseq_sfc[k]=='\0'
		){
	      Opp=treatISBD(fld,j,spi,k);
			    
	      if (Opp==FALSE)
		break;
	    }
	    else 
	      if (/*subfield is the last in the field*/
		  j==dtFld->ArrayLength-1)
		{}
	      else 
		if (		/*subseq=='*' */
		    spi->subseq_sfc[k]=='*'){
		  Opp=treatISBD(fld,j,spi,k);
		  if (Opp==TRUE)
		    break;
		}
		else
		  if (/*subseq equals the following subfield code*/
		      dtFld->subFieldCodes[j+1]==spi->subseq_sfc[k]
		      )
		    {
		      Opp=treatISBD(fld,j,spi,k);
		      if (Opp==FALSE)
			break;
		    }

	  } /*for all occurences*/
	}
      }	/*subFields in field*/
    }/*if itbl[]!=NULL*/
  }/*all fields*/
}/*FUNCTION*/

/* * * * * * * * * * * * * * * * *
 *                               *
 *      OUTPUT  FUNCTIONS        *
 *                               *
 * * * * * * * * * * * * * * * * */

char *fieldToString_privateTag(field *fld){
/*Function used when the private tag
is created.*/

  int lgt=0;
  char *str;


  if (fld->datafield==(dataField *)NULL){
  /*Probably never applicable because no one
   uses internal control fields.*/
    lgt= strlen(fld->ctrlfield->data)+1;
    
    str=(char *)calloc(lgt,sizeof(char));
    if (str==NULL){
      if (ONETK_WriteLogFile(fildes, "fieldToString()  malloc failed)")==-1){
	/**Error handle**/
	longjmp(marcconv_env, 1);
	
      }
      /*
       * exit(1);
       */
      longjmp(marcconv_env, 1);
    }

    str=strcpy(str,fld->ctrlfield->data);
    memset(str+lgt-1,'\0',1);
    return str;
  }
  else{
    /*Treat data field*/
    int i ;
    dataField *dtfld;
    char *substr;
    char *str, *aux;
    
    dtfld=fld->datafield;
    str=(char *)calloc(MAX_RECORD_LENGTH,sizeof(char));
    if (str==NULL){
      	char m[200];
	sprintf(m, 
		"fieldToString_privateTag() Malloc failed. No more memory?\n");
	if (ONETK_WriteLogFile(fildes, m)==-1){
	  /**Error handle**/
	  longjmp(marcconv_env,1);
	}
	longjmp(marcconv_env,1);
    }

    /*The private-field conversion sets the original tag first in*/
    strcpy(str, dtfld->tag) ;
    
    /*Write Indicators*/
    memset (str+3,dtfld->indicators[1],2) ;
/*    aux=(char *)memchr(str,'\0',10);
    memset(aux,dtfld->indicators[1],1);
    aux=(char *)memchr(str,'\0',10);  
    memset(aux,dtfld->indicators[2],1);
*/
    /*SUBFIELDS*/
    for(i=0;i<dtfld->ArrayLength;i++){

      lgt=lgt+strlen(dtfld->subFields[i]->data)+3;/*Include the temporary NULL terminator*/
      substr=(char *)malloc(3*sizeof(char));/**outside of loop**/
      memset(substr,'\0',3);
      memset(substr,'$',1);
      memset(substr+1,dtfld->subFieldCodes[i],1);

      strncat(str, substr, 2);
      free(substr);
      str=strcat(str,dtfld->subFields[i]->data);
      /*str=realloc(str, (strlen(str)+1)*sizeof(char));*/
    } 
    return str;
  }

}
char *fieldToStringOutput(field *fld
#ifdef MARC_DEBUG
			  , char *sutrs
#endif
)
{

/*Function used: 
  when outputting the converted record (to-record)
  when outputting the original from-record for debugging*/
  int tmplgt=0;
  int lgt=0;
  char *str;
  
  if (fld->datafield==(dataField *)NULL){

    if ( !fld->ctrlfield ) {
        ONETK_WriteLogFile(fildes, "fieldToStringOutput()  NULL Ctrl field");
	longjmp(marcconv_env, 1);
    }

    /*Treat cntrol field*/
    if (fld->ctrlfield->data){
      lgt= strlen(fld->ctrlfield->data)+20;
    
    str=(char *)calloc(lgt+2,sizeof(char));
    if (str==NULL){
      if (ONETK_WriteLogFile(fildes, "fieldToStringOutput()  malloc failed")==-1){
	/**Error handle**/
      }
      /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
    }
/*m*/


      if (fld->ctrlfield->data!=NULL){
#ifdef MARC_DEBUG
	tmplgt=strlen(sutrs);
	if(tmplgt+3+strlen(fld->ctrlfield->data)>=RCD_SUTRS_LENGTH){
	  char m[200];
	  sprintf(m, 
		  "size of RCD_SUTRS_LENGTH insufficient (datastruct.h)\n");
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  longjmp(marcconv_env,1);
	}
	sprintf(sutrs+strlen(sutrs),"%s %s\n",fld->ctrlfield->tag, fld->ctrlfield->data);
#endif
	/*m*/
	
	str=strcpy(str,fld->ctrlfield->data);
	memset(str+strlen(str),END_OF_FIELD,1);
	memset(str+strlen(str),'\0',1);
	return str;
      }
      else return NULL;
    } 
    else return NULL;
  }
  else{
    /*
     * Treat data field
     */
    int i ;
    dataField *dtfld;
    char *substr;
    char *aux;
    
    dtfld=fld->datafield;
#ifdef MARC_DEBUG
    sprintf(sutrs+strlen(sutrs),
	    "%s: ", 
	    dtfld->tag);
    
    if (readTexts==TRUE && dtfld->text!=NULL){
      if(strcmp(dtfld->tag,"999")==0){
	sprintf(sutrs+strlen(sutrs),
	      "PRIVATE TAG - NO DEFINITION\n");
      }
      else{
	sprintf(sutrs+strlen(sutrs),
	      "%s\n  ", 
	      dtfld->text);
      }
    }
    else{
      sprintf(sutrs+strlen(sutrs),
	      "\n " 
	      );
    }
    sprintf(sutrs+strlen(sutrs),
	    "indicators=%c%c\n", 
	    dtfld->indicators[1], 
	    dtfld->indicators[2]);
#endif /**MARC_DEBUG*/    
    str=(char *)calloc(MAX_RECORD_LENGTH,sizeof(char));
    if (str==NULL){
      ONETK_WriteLogFile(fildes, "fieldToStrOutput - malloc failed. Heap probably exhausted");
      longjmp(marcconv_env, 1);
      
    }

    /*INDICATORS*/
    aux=(char *)memchr(str,'\0',10);
    memset(aux,dtfld->indicators[1],1);
    aux=(char *)memchr(str,'\0',10);  
    memset(aux,dtfld->indicators[2],1);

    /*SUBFIELDS*/
    for(i=0;i<dtfld->ArrayLength;i++){
#ifdef MARC_TEST_VARIABLES
      globdtFld=dtfld;
      globInt1=i;
#endif /*MARC_TEST_VARIABLES*/
#ifdef MARC_DEBUG
      sprintf(sutrs+strlen(sutrs),
	      "  %c%c %s\n",
	      '$',dtfld->subFieldCodes[i],dtfld->subFields[i]->data);
#endif /*MARC_DEBUG*/
      lgt=lgt+strlen(dtfld->subFields[i]->data)+3;/*Include the temporary NULL terminator*/
      substr=(char *)malloc(3*sizeof(char));/**outside of loop**/
      memset(substr,'\0',3);
      memset(substr,START_OF_SUBFIELD,1);
      memset(substr+1,dtfld->subFieldCodes[i],1);

      strncat(str, substr,2);
      free(substr);
      if (strlen(dtfld->subFields[i]->data)>0)
	strncat(str,dtfld->subFields[i]->data,strlen(dtfld->subFields[i]->data));
      
    }
    memset(str+strlen(str),END_OF_FIELD,1);
    str=(char *)realloc(str, (10+strlen(str))*sizeof(char));
    return str;
  }

}

char *output2709Record(record *rcd
#ifdef MARC_DEBUG
		       ,char *rcd_sutrs /*transfered allocated*/
#endif
)
{

  field *fld_in,  *fld_out;
  char *str2709;
  char *directorystr;
  char *datastr;

  int i, tmpsize, size2709, dirlength, rcdDataLength, totalRecordLength, dataAddress, kk;
  char  *pos , *length, *tag, *totlgt, *dataadrlgt;

  
/*From NSU*/
  updateRecordFieldLengths(rcd);
  updateRecordDataLength(rcd);
  if (rcd->fields[0]!=NULL && 
	  rcd->fields[0]->ctrlfield !=NULL && 
	  rcd->fields[0]->ctrlfield->data!=NULL)
/*end From NSU*/

  memcpy(rcd->leader, 
	 rcd->fields[0]->ctrlfield->data,
	 LEADER_LENGTH);

  /*
   * Initializing directory string:
   * 000 is counted out (leader)
   */

  dirlength=12*(rcd->ArrayLength-1)+2; /*including End of 
					 field and NULL 
					 termination*/
  directorystr=(char *)calloc(dirlength+240,sizeof(char));
  
  /*Initializing data string*/
  datastr
    =(char *)malloc((rcd->dataLength+2) *sizeof(char));
  memset(datastr,'\0', rcd->dataLength+2);
#ifdef MARC_DEBUG
kk=strlen(rcd_sutrs);	 
  sprintf(rcd_sutrs+strlen(rcd_sutrs),"LEADER: %s\n",rcd->leader); kk=strlen(rcd_sutrs);
#endif
  /**OMIT "000" FIELD FOE THE TIME BEING!**/
  for (i=1;i<rcd->ArrayLength;i++){
    char *fieldstr;
    /*Convert current position to string(for the directory)*/
    pos=itoalgt(strlen(datastr),5);

    fld_out=rcd->fields[i];
    fieldstr=fieldToStringOutput(fld_out
#ifdef MARC_DEBUG
				 , rcd_sutrs
#endif
				 );
#ifdef MARC_DEBUG
    kk=strlen(rcd_sutrs);
#endif
    datastr=strncat(datastr, fieldstr,strlen(fieldstr));
    
    /*Convert field length to string (for the directory)*/
    length=itoalgt(strlen(fieldstr),4);
    if (fieldstr!=NULL)
      free(fieldstr);
    directorystr=strncat(directorystr,rcd->tags[i],3);
    directorystr=strncat(directorystr,length,4);
    directorystr=strncat(directorystr,pos,5);
    free(length);
    free(pos);

  }
  /*CLOSE THE RECORD DATA STRING */
  memset(directorystr+dirlength-2, END_OF_FIELD,1);
  memset(datastr+strlen(datastr),END_OF_RECORD,1);
  memset(datastr+strlen(datastr),'\0',1);/*TERMINATE*/

  /*CONCATENATE THE LEADER, DIRECTORY and RECORD DATA STRINGS*/
  size2709=strlen(rcd->leader)+strlen(directorystr)+strlen(datastr)+4;
  str2709=(char *)malloc(size2709*sizeof(char));
  memset(str2709, '\0',size2709);
  str2709=strcat(str2709,rcd->leader);/**For now - not converted**/
  str2709=strcat(str2709,directorystr);
  str2709=strncat(str2709,datastr, strlen(datastr));

  /*Update total record length in the beginning of the leader*/
  totalRecordLength=strlen(str2709);
  totlgt=itoalgt(totalRecordLength,5);
  memcpy(str2709,totlgt,5);
  free(totlgt);

  /*Update base address of data in posision 12 of leader*/
  dataAddress=
    LEADER_LENGTH 
      + strlen(directorystr); /*including END_OF_FIELD*/
  dataadrlgt=itoalgt(dataAddress,5);
  memcpy(str2709+12,dataadrlgt,5);
  free(dataadrlgt);	


  free(directorystr);
  free(datastr);
  return str2709; 
}









