       /* * * * * * * * * * * * * * * * *    
        *  MARCconv        Version 2.3  * 
        *   File:        readTables.c   * 
        *     Reading lookup tables    * 
        *       from  ascii files       * 
        *    Last update: 28/01/1997    * 
        *      Michael S. Preminger     * 
        *      BRODD, OSLO COLLEGE      * 
        *        THE ONE PROJECT        * 
        * * * * * * * * * * * * * * * * */

#include <setjmp.h>
#include "MARCproto.h"
extern jmp_buf marcconv_env;

/** to do: 
 * IN EACH INPUT FILE PLACE INTEGERS
 * FOR ALL OF THE "LONGEST" VALUES IN datastruct.h,
 * CHECKING THOSE AGAINST THE SET VALUES, AND PRODUCING
 * AN ERROR MESSAGE FOR EACH OVERFLOW. 
 **/

/* * * * * * * * * * * * * * * * *
 *                               *
 *      UTILITY FUNCTIONS        *
 *                               *
 * * * * * * * * * * * * * * * * */


char mygetc(FILE *fl){
  int ch;
  ch=fgetc(fl);
  while( ch=='\n' || ch==' ' )
    ch=fgetc(fl);

  return ch;
}


itemType checkNextItem(FILE *fl)
{
  itemType i_type;
  int ch;
  int i,j;
  ch=fgetc(fl);
  while (ch==' ' || ch=='\n'){
    if (feof(fl)){
      i_type=endOfFile;
      return i_type;
    }
    ch=fgetc(fl);
  }
  
  if (ch==START_VALUE)
    i_type=enclosedItem;
  else
    i_type=openItem;

  ungetc(ch,fl);
  return i_type;
}

char *getFileName(char *from, char *to, char *type){
  /** to do
   * This function should parse the to/from format names 
   * given by the running application, and return the correct
   * file names of the conversion tables
   **/
  char *fname=(char *)calloc(11, sizeof(char));
  strncpy (fname, from, 3);
  strncat (fname, to, 3);
  
  if (strcmp(type, "control")==0)
    strcat(fname, ".ctl");
  else
    if (strcmp(type, "data")==0)
      strcat(fname, ".vdt");
    else 
      if (strcmp(type, "decision")==0)
	strcat(fname, ".dcs");
      else 
	if (strcmp(type, "indicator")==0)
	  strcat(fname, ".ind");
	else 
	  if (strcmp(type, "conditional")==0)
	    strcat(fname, ".cnd");
	  else
	    if (strcmp(type, "prefix")==0)
	      strcat(fname, ".pfx");
	    else
	      if (strcmp(type, "xiferp")==0)
		strcat(fname, ".xfp");
	      

  strtolower( fname );
  return fname;
}

dataItem *readDataItem(FILE  *fl){

  /*
   * CALLED WHEN AN ITEM {***} IS EXPECTED, AND READS
   * A SINGLE CHARACTER, A CHARACTER STRING OR AN INTEGER
   */

  char ch, prev=(char)NULL;
  dataItem *tmpDI;
  tmpDI=(dataItem *)calloc(1,sizeof(dataItem));
  tmpDI->item = (char *)calloc(MAX_ITEM_LENGTH_IN_TABLE_FILE,sizeof(char));

  tmpDI->length=0;
  ch=fgetc(fl);

  /*
   * Discard leading rubish
   */
  while (!(ch==START_VALUE && prev!='\\')){
    if (feof(fl))
      return NULL;
    prev=ch;
    ch=fgetc(fl);
  }

  /*
   * here comes some
   * useful stuff
   */
  prev=ch;
  ch=fgetc(fl);			/*read first character*/
  while (!(ch==END_VALUE && prev != '\\')){
    if (tmpDI->length==MAX_ITEM_LENGTH_IN_TABLE_FILE){
      char *m;
      m=(char *)malloc(300*sizeof(char));
      sprintf(m,"MAX_ITEM_LENGTH_IN_TABLE_FILE too short in file datastruct.h, or table file with too long an item\n");
      if (ONETK_WriteLogFile(fildes, 
			     m
			     )==-1){
	/**Error handle**/
      }
      free(m);
      /*
       * exit(1);
       */
      longjmp(marcconv_env, 1);

    }
    if (feof(fl))
      return NULL;
    memset(tmpDI->item+tmpDI->length, ch,1);
    tmpDI->length++;
    prev=ch;
    ch=fgetc(fl);
  }

  if (tmpDI->length == 0){
    free (tmpDI->item);
    free (tmpDI);
    return  NULL;
  }

  /*
   * reduce the allocated size to
   * the size needed. These data are going
   * to reside in memory all day long!
   */
  tmpDI->item=realloc(tmpDI->item, 
		      (1+tmpDI->length)*sizeof(char));
  return tmpDI;
}

char readCharacterItem(FILE *fl){
  dataItem *DI;
  char ch;
  DI=readDataItem(fl);
  if (DI==NULL){
    return '\0';
  }
  else{
    if (DI->length!=1){
      char *m;
      m=(char *)malloc(200*sizeof(char));
       sprintf(m,"readCharacterItem: wrong character Value '%s' from file\n", DI->item  );
      if (ONETK_WriteLogFile(fildes, 
			     m
			     )==-1){
	/**Error handle**/
      }
      free(m);
      /*
	   * exit(1);
	   */
      longjmp(marcconv_env, 1);

    }
    else{
      ch=*(DI->item);
      free (DI->item);
      free(DI);
      return ch;
    } 
  }
}

int readIntegerItem(FILE *fl){
  dataItem *DI;

  DI=readDataItem(fl);
  if (DI==NULL){
    /** to do
     * No integer was supplied
     * A problem we wil have to look into!
     **/
    
    return 0;

  }
  else{
    if (isInteger(DI->item)==FALSE){
      char *m;
      m=(char *)malloc(200*sizeof(char));
      memset(m,'\0',200);
      sprintf(m,"readIntegerItem: Value from file not an integer: %s \n", DI->item  );
      if (ONETK_WriteLogFile(fildes, 
			     m
			     )==-1){
	/**Error handle**/
      }
      free(m);
      /*
       * exit(1);
       */
      free( DI->item );
      free (DI);
      longjmp(marcconv_env, 1);
      
    }
    else{
      int i;
      i=atoi(DI->item);
      free( DI->item );
      free (DI);
      return i;
    }
  }
}

char *readStringItem(FILE *fl){
  dataItem *DI;

  DI=readDataItem(fl);
  
  if (DI==NULL){
    return (char *)NULL;
  }
  else{
    if (DI->item!=NULL){
      int lgt= (int)strlen(DI->item)+20;
      char *str=(char *)malloc(lgt*sizeof(char));
      memset(str, '\0', lgt);
      strncpy(str, DI->item, strlen(DI->item));
      free(DI->item);
      free(DI);
      return str;
    }
    else{
      free(DI);
      return NULL;
    }
  }
}

/* * * * * * * * * * * * * * * * *
 *                               *
 *    CONSTRUCTION FUNCTIONS     *
 *                               *
 * * * * * * * * * * * * * * * * */


void *readSubPortionEntry(FILE *fl, subPortionTable_Control *spt_ctl, int index){
  /*read a single sub portion entry*/
  spt_ctl->fromSubField[index]=readCharacterItem(fl);
  spt_ctl->fromPosition[index]=readIntegerItem(fl);
  spt_ctl->fromLength[index]=readIntegerItem(fl);
  spt_ctl->fromValue[index]=readStringItem(fl);
  spt_ctl->toTag[index]=readStringItem(fl);
  spt_ctl->toSubField[index]=readCharacterItem(fl);
  spt_ctl->toPosition[index]=readIntegerItem(fl);
  spt_ctl->toLength[index]=readIntegerItem(fl);
  spt_ctl->toValue[index]=readStringItem(fl);
}
void *readInd1Entry(FILE *fl, PortionTable_VarDat *pt_vdt, int index){

  pt_vdt->ind1fromValue[index]=readCharacterItem(fl);
  pt_vdt->ind1function[index]=readCharacterItem(fl);
  pt_vdt->ind1toValue[index]=readCharacterItem(fl);
  if (pt_vdt->ind1toValue[index]=='\0')
    pt_vdt->ind1toValue[index]=' ';
  pt_vdt->ind1special[index]=readStringItem(fl);
  pt_vdt->ind1toInd[index]=readIntegerItem(fl);
}
void *readInd2Entry(FILE *fl, PortionTable_VarDat *pt_vdt, int index){

  pt_vdt->ind2fromValue[index]=readCharacterItem(fl);
  pt_vdt->ind2function[index]=readCharacterItem(fl);
  pt_vdt->ind2toValue[index]=readCharacterItem(fl);
  if (pt_vdt->ind2toValue[index]=='\0')
    pt_vdt->ind2toValue[index]=' ';
  pt_vdt->ind2special[index]=readStringItem(fl);
  pt_vdt->ind2toInd[index]=readIntegerItem(fl);
}
void *readSfEntry(FILE *fl, PortionTable_VarDat *pt_vdt, int index){

  pt_vdt->fromSubfieldCode[index]=readCharacterItem(fl);
  pt_vdt->fromFunction[index]=readCharacterItem(fl);
  pt_vdt->treatment_par[index]=readStringItem(fl);
  pt_vdt->toSubField[index]=readStringItem(fl);
  /*pt_vdt->toTag[index]=readStringItem(fl);*/
}

     
     
     
int readCtrlTable(char *fromFormat, 
		    char *toFormat, 
		    conversion_tbl_str *cns)
{
  int res, inchar;
  char *fname;
  int tagNumber, arrayLength;
  FILE *infile;
  PortionTable_Control *pt_CTL;
  subPortionTable_Control *spt_CTL;
  char *infileName;
  generic_table *gn_tbl;
/*
 * Barry Change comment. We want to search through the path to locate the
 * file
 */
  fname=getFileName(fromFormat, toFormat, "control");
  infileName = ONETK_FilePath(fname);
  infile=fopen(infileName, "r");
  if (infile==NULL){
	  char m[200];
	  sprintf(m, "function: readCtrlTable\n  control field conversion table file %s not found", fname);
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  longjmp(marcconv_env, 1);
	  }
	  /*
	   * exit(1);
	   */
	  return -1;

	}
  inchar=mygetc(infile);
  while (!feof(infile)){
    

    while (inchar==NEW_TAG){
      int isp=0;
      tagNumber=readIntegerItem(infile);

      if ( cns->Conversion_Table[tagNumber] == NULL)
	cns->Conversion_Table[tagNumber]=
	  (generic_table *)calloc(1,sizeof(generic_table));
      gn_tbl=cns->Conversion_Table[tagNumber];

      if (gn_tbl->portion_table_ctl== NULL) 
	gn_tbl->portion_table_ctl=(PortionTable_Control *)calloc(1, sizeof(PortionTable_Control));
      pt_CTL=gn_tbl->portion_table_ctl;
      
      inchar=mygetc(infile);
      while (inchar==NEW_CONSULTING_VALUES){ /*and not NEW_TAG*/
	int ispte=0;
	for (isp=0;isp<LONGEST_CONSULT;isp++){
	  if (pt_CTL->subPortion_CTL[isp]==NULL)
	    break;
	}
	pt_CTL->consultValue1[isp]=readStringItem(infile);
	if (pt_CTL->consultValue1[isp]==NULL
	    &&
	    (tagNumber==6 || tagNumber==7 || tagNumber==8)){
	  char m[200];
	  sprintf(m, "file: readTables.c\n  warning: consultValue1 void. in 00%d\n", tagNumber);
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	}
	pt_CTL->consultValue2[isp]=readStringItem(infile);
	if (pt_CTL->consultValue2==NULL && tagNumber==8){
	  char m[200];
	  sprintf(m, "file: readTables.c\n  warning: consultValue2 void in 008\n");
	  if(ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/	  
	  }
	}
	
	pt_CTL->subPortion_CTL[isp]=
	  (subPortionTable_Control *) calloc(1,sizeof(subPortionTable_Control));
	spt_CTL = pt_CTL->subPortion_CTL[isp];
	inchar=mygetc(infile);
	while(inchar==NEW_ENTRY){ 
	  readSubPortionEntry(infile, spt_CTL, ispte);
	  inchar=mygetc(infile);	/**mygetc(infile)**/
	  ispte++;
	}
	spt_CTL->Arraylength=ispte;
	isp++;
      }
      pt_CTL->Arraylength=isp;

    }


  }
  fclose(infile);  
  free(infileName);		/* release the memory */
  return 0;
}  
decision_table_str  *readDecisionTable(char *fromFormat, 
				       char *toFormat)
{
  int ctr=0, idcs=0, isdcs=0;
  static int inchar;
  char *fn;
  char  *tagNumber;
    int arrayLength;
  PortionTable_Decision *pt_DCS;
  FILE *infile;
  char *fnPath;
  decision_table_str *dts=(decision_table_str *)calloc(1, sizeof(decision_table_str));

/*
 * Barry Change Comment. Again, locate the file through the path mechanism
 */
  fn=getFileName(fromFormat, toFormat, "decision");
  fnPath = ONETK_FilePath(fn);
  if (fnPath==NULL){
    return NULL;
  }
  infile=fopen(fnPath, "r");
  if (infile==NULL){
    return NULL;
  }

  dts=(decision_table_str *)calloc(1,
				   sizeof(decision_table_str));

  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      isdcs=0;
      tagNumber=readStringItem(infile);

      dts->fromTags[idcs]=(char *)malloc(4*sizeof(char));
      memcpy(dts->fromTags[idcs], tagNumber, 3);
      memset(dts->fromTags[idcs]+3,'\0',1);
      inchar=mygetc(infile);
      while (inchar==NEW_PORTION_TABLE){
	int ill=0;
	PortionTable_Decision *new_pt_DCS =(PortionTable_Decision *)calloc(1,sizeof(PortionTable_Decision));
	if (isdcs>0)
	  /*tables of same tag are put in a linked list*/
	  pt_DCS->NextPT_DCS=new_pt_DCS;
	else
	  dts->Table[idcs]=new_pt_DCS;
	
	pt_DCS=new_pt_DCS;
	pt_DCS->indicator1=readCharacterItem(infile);
	pt_DCS->indicator2=readCharacterItem(infile);

	/*This one is to be changed,  to read a string item*/
	pt_DCS->material=readStringItem(infile);

	pt_DCS->subFieldCode=readCharacterItem(infile);
	pt_DCS->toTag=readStringItem(infile);
	pt_DCS->defaultToTag=readStringItem(infile);
	inchar=mygetc(infile);
	while (inchar==NEW_LOW_LEVEL_ENTRY){
	  if (ill>LONGEST_DECISION_TABLE_LOW_LEVEL){
	    char m[200];
	    sprintf(m, "file: readTables.c\n  warning: consultValue2 void in 008\n");
	    if(ONETK_WriteLogFile(fildes, m)==-1){
	      /**Error handle**/	  
	    }
	    longjmp(marcconv_env, 1);
	  }
	  pt_DCS->lowerLevelTable_txt[ill]=readStringItem(infile);
	  pt_DCS->lowerLevelTable_toTag[ill] = readStringItem(infile);
	  ill++;
	 inchar=mygetc(infile); 
	}
	pt_DCS->ArrayLength=ill;
	isdcs++;
      }/*while (inchar==NEW_PORTION_TABLE*/
      idcs++;
    }/*while (inchar==NEW_TAG)*/
    dts->ArrayLength=idcs;
  }/*while (inchar!=EOF)*/
  free(fnPath); /* free this memory */
  return dts;
}

indicator_table_str  *readIndicatorTable(char *fromFormat, 
				       char *toFormat)
{
  int ctr=0, iind=0, isind=0, rescomp;
  static int inchar;
  char *fn;
  char  *tagNumber;
    int arrayLength;
  PortionTable_Indicator *pt_IND;
  FILE *infile;
  char *fnPath;
  indicator_table_str *its;
  char toInd;
  char *rgExp;
  
/*
 * Barry Change Comment. Again, locate the file through the path mechanism
 */
  fn=getFileName(fromFormat, toFormat, "indicator");
  fnPath = ONETK_FilePath(fn);
  if (fnPath == NULL) {
    return NULL;
  }
  infile=fopen(fnPath, "r");
  if (infile==NULL){
    return NULL;
  }

  its=(indicator_table_str *)calloc(1,
				   sizeof(indicator_table_str));

  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      isind=0;
      its->fromTags[iind]=readStringItem(infile);
      its->toTags[iind]=readStringItem(infile);
      if (its->toTags[iind]!=NULL){
	rgExp=toRegExp(its->toTags[iind]);
	its->toTags_compExp[iind]=
	  (regex_t*)calloc(1,sizeof(regex_t));
	rescomp=regcomp(its->toTags_compExp[iind],
			rgExp,
			REG_NOSUB);
	free(rgExp);
	if (rescomp!=0){
	  char m[200];
	    sprintf(m, 
		    " readIndicatorTable: Compilation of regexp %s failed\n", 
		    its->toTags[iind]
		    );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	    
	  }
	  
	}
	
      }
      its->whichInds[iind]=readIntegerItem(infile);
      if (its->whichInds[iind]!=1 && its->whichInds[iind]!=2){
		  char m[200];
	    sprintf(m, 
		    " readIndicatorTable: whichInds parameter must be either 1 or 2, nor %d. Probably a table error.\n", 
		    its->whichInds[iind]
		    );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	    
	  }

      }
      inchar=mygetc(infile);
      while (inchar==NEW_PORTION_TABLE){
	int ill=0;
	/*?????????*/
	PortionTable_Indicator *new_pt_IND
	  =(PortionTable_Indicator *)calloc(1,sizeof(PortionTable_Indicator));
	if (isind>0)
	  /*tables of same tag/indicator are put in a linked list*/
	  pt_IND->NextPT_IND=new_pt_IND;
	else
	  its->Table[iind]=new_pt_IND;
	
	pt_IND=new_pt_IND;
	pt_IND->checkField=readStringItem(infile);
	if (pt_IND->checkField!=NULL){
	  rgExp=toRegExp(pt_IND->checkField);
	  pt_IND->checkField_compExp=
	    (regex_t*)calloc(1,sizeof(regex_t));
	  rescomp=regcomp(pt_IND->checkField_compExp,
			  rgExp,
			  REG_NOSUB);
	  free(rgExp);
	  if (rescomp!=0){
	    char m[200];
	    sprintf(m, 
		    " readIndicatorTable: Compilation of regexp %s failed\n", 
		    pt_IND->checkField
		    );
	    if (ONETK_WriteLogFile(fildes, m)==-1){
	      /**Error handle**/
	      
	    }
  
	    }
	  
	}
	
	pt_IND->checkSubField=readCharacterItem(infile);
	pt_IND->condition=readCharacterItem(infile);
	pt_IND->parameter=readStringItem(infile);
	pt_IND->nonDefaultToInd=readCharacterItem(infile);
	pt_IND->defaultToInd=readCharacterItem(infile);
	inchar=mygetc(infile);
	while (inchar==NEW_LOW_LEVEL_ENTRY){
	  pt_IND->lowerLevelTable_txt[ill]=readStringItem(infile);
	  if (pt_IND->lowerLevelTable_txt[ill]!=NULL){
	    if (strncmp(pt_IND->lowerLevelTable_txt[ill], "^", 1)!=0)
	      rgExp=toRegExp(pt_IND->lowerLevelTable_txt[ill]);
	    else
	      rgExp=toRegExp_firstWord(pt_IND->lowerLevelTable_txt[ill]);
	    pt_IND->lowerLevelTable_compExp[ill]=
	      (regex_t*)calloc(1,sizeof(regex_t));
	    rescomp=regcomp(pt_IND->lowerLevelTable_compExp[ill],
			    rgExp,
			    REG_NOSUB);
	    free(rgExp);
	    if (rescomp!=0){
	      char m[200];
	      sprintf(m, 
		      " readIndicatorTable: Compilation of regexp %s failed\n", 
		      pt_IND->lowerLevelTable_txt[ill]
		      );
	      if (ONETK_WriteLogFile(fildes, m)==-1){
		/**Error handle**/
		
	      }
	      
	    }
	    
	  }
	  
	  pt_IND->lowerLevelTable_toInd[ill] = readCharacterItem(infile);

	  if (pt_IND->lowerLevelTable_txt[ill]!=NULL){
	    pt_IND->lowerLevelTable_compExp[ill]=(regex_t*)calloc(1,sizeof(regex_t));
	    rescomp=regcomp(pt_IND->lowerLevelTable_compExp[ill],
			    pt_IND->lowerLevelTable_txt[ill],
			    REG_NOSUB);
	    if (rescomp!=0){
	      char m[200];
	      sprintf(m, 
		      " readIndicatorTable: Compilation of regexp %s failed\n", 
		      pt_IND->lowerLevelTable_txt[ill]
		      );
	      if (ONETK_WriteLogFile(fildes, m)==-1){
		/**Error handle**/

	      }
  
	    }
	  }
	  ill++;
	 inchar=mygetc(infile); 
	}
	pt_IND->ArrayLength=ill;
	isind++;
      }/*while (inchar==NEW_PORTION_TABLE*/
      iind++;
    }/*while (inchar==NEW_TAG)*/
    its->ArrayLength=iind;
  }/*while (inchar!=EOF)*/
  free(fnPath); /* free this memory */
  return its;
}
conditional_table_str  *readConditionalTable(char *fromFormat, 
				       char *toFormat)
{
  int ctr=0, icnd=0, iscnd=0;
  static int inchar;
  char *fn;
  char  *tagNumber;
  char *sf_content_regExp;
  int arrayLength, rescomp;
  PortionTable_Conditional *pt_CND;
  FILE *infile;
  char *fnPath;
  conditional_table_str *cts;
 
  
/*
 * Barry Change Comment. Again, locate the file through the path mechanism
 */
  fn=getFileName(fromFormat, toFormat, "conditional");
  fnPath = ONETK_FilePath(fn);
  if (fnPath==NULL){
    return NULL;
  }
  infile=fopen(fnPath, "r");
  free(fnPath); /* free this memory */
  if (infile==NULL){
    return NULL;
  }

  cts=(conditional_table_str *)calloc(1,
				   sizeof(conditional_table_str));

  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      iscnd=0;
      cts->fromTags[icnd]=readStringItem(infile);
      cts->toTags[icnd]=readStringItem(infile);
      cts->Table[icnd]=(PortionTable_Conditional *)calloc(1,sizeof(PortionTable_Conditional));
      inchar=mygetc(infile);
      pt_CND=cts->Table[icnd];
      while (inchar==NEW_PORTION_TABLE){
	if (iscnd==LONGEST_SUB_PORTION_CONDITIONAL){
	  char m[200];
	  sprintf(m, "LONGEST_SUB_PORTION_CONDITIONAL %d, too short\n",
		  iscnd
		  );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
#ifndef _ONETK_LIB_BUILD_
	  exit(1);
#else	  
	  longjmp(marcconv_env, 1);
#endif
	}
	
	pt_CND->fromSubFieldCode[iscnd]=readCharacterItem(infile);
	pt_CND->sf_contents[iscnd]=readStringItem(infile);
	
	if(pt_CND->sf_contents[iscnd]!=NULL){
	  pt_CND->sf_content_compExp[iscnd]=(regex_t*)calloc(1,sizeof(regex_t));

	  /*
	   * Convert to a regexp if necessary, and compile
	   * the regular expression
	   */
	  sf_content_regExp=toRegExp(pt_CND->sf_contents[iscnd]);
	  rescomp=regcomp(pt_CND->sf_content_compExp[iscnd],
			  toRegExp(pt_CND->sf_contents[iscnd]),
			  REG_NOSUB);
	  if (rescomp!=0){
	    char m[200];
	    sprintf(m, 
		    " readConditionalTable: Compilation of regexp %s failed\n", 
		    pt_CND->sf_contents[iscnd]
		    );
	    if (ONETK_WriteLogFile(fildes, m)==-1){
	      /**Error handle**/
	    
	    }
	  
	  }
	  if (sf_content_regExp != NULL)
	    free(sf_content_regExp);
	}
	pt_CND->condition[iscnd]=readCharacterItem(infile);
	pt_CND->parameter[iscnd]=readStringItem(infile);
	pt_CND->fromFunction[iscnd]=readCharacterItem(infile);
	pt_CND->toSubField[iscnd]=readStringItem(infile);
	pt_CND->treatment_par[iscnd]=readStringItem(infile);
	inchar=mygetc(infile);
	iscnd++;
      }/*while (inchar==NEW_PORTION_TABLE*/
      
      pt_CND->ArrayLength=iscnd;
      icnd++;
      
    }/*while (inchar==NEW_TAG)*/
    cts->ArrayLength=icnd;    
  }/*while (inchar!=EOF)*/

  return cts;
}

prefixing_table_str  *readPrefixTable(char *fromFormat, 
				       char *toFormat)
{
  int ctr=0, ipfx=0, ispfx=0;
  static int inchar;
  char *fn;
  char  *tagNumber;
  char *sf_content_regExp;
  int arrayLength, rescomp;
  PortionTable_Prefix *pt_PFX;
  FILE *infile;
  char *fnPath;
  prefixing_table_str *pts;
 
  
/*
 * Barry Change Comment. Again, locate the file through the path mechanism
 */
  fn=getFileName(fromFormat, toFormat, "prefix");
  fnPath = ONETK_FilePath(fn);
  if (fnPath==NULL){
    return NULL;
  }
  infile=fopen(fnPath, "r");
  free(fnPath); /* free this memory */
  if (infile==NULL){
    return NULL;
  }

  pts=(prefixing_table_str *)calloc(1,
				   sizeof(prefixing_table_str));

  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      ispfx=0;
      pts->fromTags[ipfx]=readStringItem(infile);
      pts->fromInd1[ipfx]=readCharacterItem(infile);
      pts->fromInd2[ipfx]=readCharacterItem(infile);
      pts->Table[ipfx]=(PortionTable_Prefix *)calloc(1,sizeof(PortionTable_Prefix));
      inchar=mygetc(infile);
      pt_PFX=pts->Table[ipfx];
      while (inchar==NEW_PORTION_TABLE){
	if (ispfx==LONGEST_PREFIX_PORTION){
	  char m[200];
	  sprintf(m, "LONGEST_SUB_PORTION_PREFIX %d, too short\n",
		  ispfx
		  );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);

	}

	pt_PFX->fromSubFieldCode[ispfx]=readCharacterItem(infile);
	pt_PFX->sf_contents[ispfx]=readStringItem(infile);
	if(pt_PFX->sf_contents[ispfx]!=NULL){
#ifdef SPLIT_REGEX
	  pt_PFX->sf_content_compExp[ispfx]=(struct re_pattern_buffer *)calloc(1,sizeof(struct re_pattern_buffer));
	  
#else /*SPLIT_REGEX*/
	  pt_PFX->sf_content_compExp[ispfx]=(regex_t *)calloc(1,sizeof(regex_t));
#endif/*SPLIT_REGEX*/
	  /*
	   * Convert to a regexp if necessary, and compile
	   * the regular expression
	   */
	  sf_content_regExp=toRegExp(pt_PFX->sf_contents[ispfx]);
#ifdef SPLIT_REGEX
	  rescomp=re_compile_pattern(sf_content_regExp,
				     strlen(sf_content_regExp),
				     pt_PFX->sf_content_compExp[ispfx]);
			     
	  pt_PFX->sf_content_compExp[ispfx]->regs_allocated=REGS_UNALLOCATED;	     

#else /*SPLIT_REGEX*/
	  rescomp=regcomp(pt_PFX->sf_content_compExp[ispfx],
			  sf_content_regExp,
			  REG_NOSUB);
 
#endif /*SPLIT_REGEX*/
	  if (rescomp!=0){
	    char m[100];
	    sprintf(m, 
		    " readPrefixTable: Compilation of regexp %s failed\n", 
		    pt_PFX->sf_contents[ispfx]
		    );
	    if (ONETK_WriteLogFile(fildes, m)==-1){
	      /**Error handle**/
	    
	    }
	  
	  }
	  if (sf_content_regExp != NULL)
	    free(sf_content_regExp);
	}

	pt_PFX->fromFunction[ispfx]=readCharacterItem(infile);
	pt_PFX->toField[ispfx]=readStringItem(infile);
	pt_PFX->treatment_par[ispfx]=readStringItem(infile);
	inchar=mygetc(infile);
	ispfx++;
      }	/*while (inchar==NEW_PORTION_TABLE*/
      
      pt_PFX->ArrayLength=ispfx;
      ipfx++;
      
    }/*while (inchar==NEW_TAG)*/
    pts->ArrayLength=ipfx;    
  }/*while (inchar!=EOF)*/

  return pts;
}

xiferping_table_str  *readXiferpTable(char *fromFormat, 
				      char *toFormat)
{
  int ctr=0, ixfp=0, isxfp=0;
  static int inchar;
  char *fn;
  char  *tagNumber;
  char *sf_content_regExp;
  int arrayLength, rescomp;
  PortionTable_Xiferp *pt_XFP;
  FILE *infile;
  char *fnPath;
  xiferping_table_str *xts;
  char *tag_regExp;
  
  /*
   * Barry Change Comment. Again, locate the file through the path mechanism
   */
  fn=getFileName(fromFormat, toFormat, "xiferp");
  fnPath = ONETK_FilePath(fn);
  if (fnPath==NULL){
    return NULL;
  }
  infile=fopen(fnPath, "r");
  free(fnPath);			/* free this memory */
  if (infile==NULL){
    return NULL;
  }

  xts=(xiferping_table_str *)calloc(1,
				    sizeof(xiferping_table_str));

  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      isxfp=0;
      xts->fromTags[ixfp]=readStringItem(infile);
      xts->ToTags_text[ixfp]=readStringItem(infile);
      if (xts->ToTags_text[ixfp]!=NULL){
	xts->ToTags_compExp[ixfp]=(regex_t *)calloc(1,sizeof(regex_t));

	/*
	 * Convert to a regexp if necessary, and compile
	 * the regular expression
	 */
	tag_regExp=toRegExp(xts->ToTags_text[ixfp]);
	
	rescomp=regcomp(xts->ToTags_compExp[ixfp],
			tag_regExp,
			REG_NOSUB);
	
	if (rescomp!=0){
	  char m[100];
	  sprintf(m, 
		  " readXiferpTable: Compilation of regexp %s failed\n", 
		  xts->ToTags_text[ixfp]
		  );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	    
	  }
	  
	}
	if (tag_regExp != NULL)
	  free(tag_regExp);
      }
      
      xts->Table[ixfp]=(PortionTable_Xiferp *)calloc(1,sizeof(PortionTable_Xiferp));
      inchar=mygetc(infile);
      pt_XFP=xts->Table[ixfp];
      while (inchar==NEW_PORTION_TABLE){
	if (isxfp==LONGEST_PREFIX_PORTION){
	  char m[200];
	  sprintf(m, "LONGEST_SUB_PORTION_PREFIX %d, too short\n",
		  isxfp
		  );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	  
	}
	
	pt_XFP->fromSubFieldCode[isxfp]=readCharacterItem(infile);
	pt_XFP->fromFunction[isxfp]=readCharacterItem(infile);
	pt_XFP->toSFCodes[isxfp]=readStringItem(infile);
	pt_XFP->treatment_par[isxfp]=readStringItem(infile);
	inchar=mygetc(infile);
	isxfp++;
      }	/*while (inchar==NEW_PORTION_TABLE*/
    
      pt_XFP->ArrayLength=isxfp;
      ixfp++;
      
    } /*while (inchar==NEW_TAG)*/
    xts->ArrayLength=ixfp;    
  } /*while (inchar!=EOF)*/
  
  return xts;
}

int readVdfTable(char *fromFormat, 
		 char *toFormat, 
		 conversion_tbl_str *cns)
{
  int ctr=0, res=0;
  int inchar;
  char *fn;
  int tagNumber, arrayLength;
  Boolean special_treatment;
  PortionTable_VarDat *pt_VDT;
  FILE *infile;
  char *infilePath;

  /*
   * Barry Change Comment. Again search for file in path
   */
  fn=getFileName(fromFormat, toFormat, "data");
  infilePath = ONETK_FilePath(fn);
  infile=fopen(infilePath, "r");
  if (infile == NULL){
    char m[200];
    sprintf(m, 
	    "readVdfTable()\n  No data field conversion table provided.\n   file %s not found.",
	    fn);
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
     * exit(1);
     */
    return -1;
    
  }
  inchar=mygetc(infile);
  while (!feof(infile)){
    
    while (inchar==NEW_TAG){
      generic_table *gn_tbl, *dummy_gn_tbl=NULL;
      itemType itType;
      int isp=0;
      int treatmentValue;
      ctr++;
      tagNumber=readIntegerItem(infile);
      
      if (cns->Conversion_Table[tagNumber]!=NULL){
	char *m;
	m=(char *)malloc(300*sizeof(char));
	sprintf(m,"tables from %s to %s: field tag %d occurs in both control field and data field tables. Data field occurence ignored \n", fromFormat, toFormat, tagNumber);
	if (ONETK_WriteLogFile(fildes, 
			       m
			       )==-1){
	  /**Error handle**/
	}
	free(m);
	/*
	 * Use a dummy gn_tbl in order to properly
	 * proceed with the text file reading
	 */
	dummy_gn_tbl = (generic_table *)calloc(1,sizeof(generic_table));
	gn_tbl=dummy_gn_tbl;
      }
      else{
	cns->Conversion_Table[tagNumber]=
	  (generic_table *)calloc(1,sizeof(generic_table));
	gn_tbl=cns->Conversion_Table[tagNumber];

	/*This one reads either a 0, 1, 2, 3 or 4*/
	treatmentValue=readIntegerItem(infile);
	switch(treatmentValue){
	case NORMAL_TREATMENT:
	  gn_tbl->specialTreatment=FALSE;
	  gn_tbl->prefixTreatment=FALSE;
	  gn_tbl->xiferpTreatment=FALSE;
	  gn_tbl->copyField=FALSE;
	  break;
	case PREFIX_TREATMENT:
	  gn_tbl->specialTreatment=FALSE;
	  gn_tbl->prefixTreatment=TRUE;
	  gn_tbl->xiferpTreatment=FALSE;
	  gn_tbl->copyField=FALSE;
	  break;
	case SPECIAL_TREATMENT:
	  gn_tbl->specialTreatment=TRUE;
	  gn_tbl->prefixTreatment=FALSE;
	  gn_tbl->xiferpTreatment=FALSE;
	  gn_tbl->copyField=FALSE;
	  break;
	case XIFERP_TREATMENT:
	  gn_tbl->specialTreatment=FALSE;
	  gn_tbl->prefixTreatment=FALSE;
	  gn_tbl->xiferpTreatment=TRUE;
	  gn_tbl->copyField=FALSE;
	  break;
	case COPY_FIELD:
	  gn_tbl->specialTreatment=FALSE;
	  gn_tbl->prefixTreatment=FALSE;
	  gn_tbl->xiferpTreatment=FALSE;
	  gn_tbl->copyField=TRUE;
	  break;
	default:{
	    char *m;
	    m=(char *)malloc(300*sizeof(char));
	    sprintf(m," unknown code %c in table for tag %d in table file %s\n", treatmentValue, tagNumber, infilePath);
	    if (ONETK_WriteLogFile(fildes, 
				   m
				   )==-1){
	      /**Error handle**/
	    }
	    free(m);
	    
	  }
	  
	}
	/* if there is an enclosed item starting with a
	 * %-character, it is a tag to copy the field under in the
	 * from record
	 */




	/*
	 * If there is an enclosed item after this one,
	 * this means we do not have a to tag here - 
	 * and therefore no portion_table_vdt. The
	 * enclosed item is a text that is used when
	 * generating the private tag, and describes the
	 * function of the from tag.
	 */
	itType=checkNextItem(infile);
	if (itType==endOfFile){
	  char *m;
	  m=(char *)malloc(200*sizeof(char));
	  memset(m, '\0',200);
	  sprintf(m,"End of file character detected in file %s\n", infilePath   );
	  if (ONETK_WriteLogFile(fildes, 
				 m
				 )==-1){
	    /**Error handle**/
	  }
	  free(m);
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	  
	  
	}
      backHere:
	if (itType==enclosedItem){
	  char *it=readStringItem(infile);
	  if (*it=='%'){
	    gn_tbl->copyTag=(char *)calloc(4, sizeof(char));
	    strncpy(gn_tbl->copyTag,it+1,3);
	    free(it);
	    itType=checkNextItem(infile);

	    /*Terrible, but saves some code and avoids some confusion*/
	    goto backHere;
	  }
	  if (readTexts==TRUE){
	    gn_tbl->defaultText=(char *)calloc(1+strlen(it), sizeof(char));
	    strncpy(gn_tbl->defaultText, it, strlen(it));
	    free(it);
	  }
	  /*
	   * and the next inchar=mygetc(..)
	   * will give us a NEW_TAG again
	   */

	}
	else{
	  gn_tbl->portion_table_vdt=
	    (PortionTable_VarDat *)
	    calloc(1,
		   sizeof(PortionTable_VarDat)); 
	  
	  pt_VDT=gn_tbl->portion_table_vdt;
	}
	inchar=mygetc(infile);
	while (inchar==NEW_PORTION_TABLE){

	  int i1=0, i2=0, sf=0;
	  ctr++;
	  if (isp > 0) {
	    pt_VDT->NextPT_DT=(PortionTable_VarDat *)calloc(1,sizeof(PortionTable_VarDat));
	    pt_VDT=pt_VDT->NextPT_DT;
	  }
	  pt_VDT->toTag=readStringItem(infile);
	  /* 
	   * Read explaining text if 
	   * required and available
	   */
	  itType=checkNextItem(infile);
	  if (itType==endOfFile){
	    char m[200];
	    sprintf(m,"End of file character detected in file %s\n", infilePath   );
	    if (ONETK_WriteLogFile(fildes, 
				   m
				   )==-1){
	      /**Error handle**/
	    }
	    /*
	     * exit(1);
	     */
	    longjmp(marcconv_env, 1);
	  
	  
	  }
	  if (itType==enclosedItem)
	    if (readTexts==TRUE)
	      pt_VDT->text=readStringItem(infile);
	
	  inchar=mygetc(infile);
	  while(inchar==IND1_ENTRY){ 
	    ctr++;
	    readInd1Entry(infile, pt_VDT, i1);
	    inchar=mygetc(infile);
	    i1++;
	  }
	  pt_VDT->ind1ArrayLength=i1;
	  while(inchar==IND2_ENTRY){
	    ctr++;
	    readInd2Entry(infile, pt_VDT, i2);
	    inchar=mygetc(infile);
	    i2++;
	  }
	  pt_VDT->ind2ArrayLength=i2;
	  while(inchar==SF_ENTRY){
	    ctr++;
	    readSfEntry(infile, pt_VDT, sf);
	    inchar=mygetc(infile);
	    sf++;
	  }
	  pt_VDT->ArrayLength=sf;
	  isp++;
	}
      }
      if (dummy_gn_tbl!=NULL)
	free(dummy_gn_tbl);
    }
    fclose(infile);
    if (infilePath)
      {
	free(infilePath);
      }
    return 0;
  }  
}


void *readISBDTable(PortionTable_ISBD *Isbd_Tbl[])
{
  FILE *infile;
  static int inchar;
  char *infilePath;

  int tagNumber, arrayLength;

  /*
   * Barry Change Comment. Again we want to look for this file in the path
   */
  infilePath = ONETK_FilePath("usmarc.isb");
  if (infilePath==NULL){
    char m[200];
    sprintf(m, "readISBDTable(): unable to find file usmarc.isb\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    infile=NULL;
  }
  else{
    infile=fopen(infilePath, "r");
  }
  if (infile==NULL){
    char m[200];
    sprintf(m, "readISBDTable(): unable to read file usmarc.isb\nISBD punctuatuion of USMARC records will not be treated\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }

  }
  else{
    inchar=mygetc(infile);
    while (!feof(infile)){
      while (inchar==NEW_TAG){
	PortionTable_ISBD *pt_ISBD;

	tagNumber=readIntegerItem(infile);
	pt_ISBD=newPortionTable_ISBD(tagNumber, Isbd_Tbl);
	pt_ISBD->ArrayLength=0;
	
	inchar=mygetc(infile);
	while (inchar==NEW_SUB_PORTION_TABLE_ISBD){
	  subPortionTable_ISBD *spt_ISBD;

	  spt_ISBD=newSubPortionTable_ISBD(pt_ISBD);
	  spt_ISBD->ArrayLength=0;

	  pt_ISBD->subFieldCodes[pt_ISBD->ArrayLength]=
	    readCharacterItem(infile);

	  inchar=mygetc(infile);
	  while (inchar==NEW_ENTRY){
	    spt_ISBD->subseq_sfc[spt_ISBD->ArrayLength]=
	      readCharacterItem(infile);
	    spt_ISBD->function[spt_ISBD->ArrayLength]=
	      readCharacterItem(infile);
	    spt_ISBD->treatment_par[spt_ISBD->ArrayLength]=
	      readStringItem(infile);
	    spt_ISBD->Punctuation[spt_ISBD->ArrayLength]=
	      readStringItem(infile);

	    spt_ISBD->ArrayLength++;
	    inchar=mygetc(infile);
	  } /*NEW_ENTRY*/
	  pt_ISBD->ArrayLength++;
	
	} /*NEW_PORTION_TABLE*/

      }	/*NEW_TAG*/
    
    } /*EOF*/
  }
  if (infilePath)
    {
      free(infilePath);
    }
}
#ifdef MARC_MULTI_TABLE
int findLookupTable(char *f,
		    char *t){
  int tmp;
  for (tmp=0;tmp<lt_ArrayLength;tmp++){
    if (strcmp(f,lt[tmp]->fromFormatName)==0
	&&
	(strcmp(t,lt[tmp]->toFormatName)==0))
      return tmp;
  }
  {
    char m[200];
    sprintf(m,"findLookupTable()\n Combination not available: %s %s\n check the file convsets.txt",
	    f,
	    t);
    if (ONETK_WriteLogFile(fildes, 
			   m
			   )==-1){
      /**Error handle**/
    }
    /*
     * exit(1);
     */
    longjmp(marcconv_env, 1);
  }
  
}


lookup_table *readConversionSet(char *fromFormat,
				char *toFormat){
  int q=0, res=0;
  lookup_table *tmp_lt;
  tmp_lt=(lookup_table *)calloc(1,sizeof(lookup_table));
  tmp_lt->cns=(conversion_tbl_str *)calloc(1,sizeof(conversion_tbl_str));
  tmp_lt->fromFormatName=fromFormat;
  tmp_lt->toFormatName=toFormat;
  /*INITIALIZE*/
  for (q=0;q<1000;q++)
    tmp_lt->cns->Conversion_Table[q]=NULL;
  res=readVdfTable(fromFormat, toFormat,tmp_lt->cns) ;
  if (res!=0) return NULL;
  res=readCtrlTable(fromFormat, toFormat, tmp_lt->cns) ;
  if (res!=0) return NULL;
  tmp_lt->dts=readDecisionTable(fromFormat, toFormat);  
  tmp_lt->its=readIndicatorTable(fromFormat, toFormat);
  tmp_lt->cts=readConditionalTable(fromFormat, toFormat);
  tmp_lt->pts=readPrefixTable(fromFormat, toFormat);
  tmp_lt->xts=readXiferpTable(fromFormat, toFormat);
  
  return tmp_lt;
  
}
#ifndef _ONETK_LIB_BUILD_
int TABREAD( void ){
/*OPEN FILE THAT LISTS THE TABLES TO BE READ TODAY*/
  int i, pos, ch, res;
  char *ffname;
  char *ttname;
  char *infilePath;
  Boolean UsMaRc=FALSE;
  FILE *tablesDeJour;
 
  /*Initializing lt[] GLOBAL*/
  for (i=0;i<MAX_TABLES;i++)
    lt[i]=NULL;

 
  infilePath = ONETK_FilePath("convsets.txt");
  tablesDeJour=fopen(infilePath, "r");
  free(infilePath);
  while(1==1){
    if (feof(tablesDeJour))
      break;

    ch=mygetc(tablesDeJour);
    while(ch!=START_VALUE&&!feof(tablesDeJour)){ 
      ch=mygetc(tablesDeJour);
    }
    if (feof(tablesDeJour))
      break;
    ungetc(ch,tablesDeJour);
    ffname=readStringItem(tablesDeJour);
    ttname=readStringItem(tablesDeJour);
/*    pos=findLookupTable(ffname, ttname);*/

    /* 
     *  We need to know if USMARC is involved
     * in any of the tables if we should load
     * the ISBD table
     */
    if (UsMaRc==FALSE)
      if (strcmp(ffname,"USMARC")==0 
	  ||
	  strcmp(ttname,"USMARC")==0)
	UsMaRc=TRUE;

    lt[lt_ArrayLength]=readConversionSet(ffname,ttname);

    if (lt[lt_ArrayLength]==NULL){
          char m[200];
	  sprintf(m, "TABREAD: unable to read conversion set for %s to %s\n",
		  ffname,
		  ttname);
	if (ONETK_WriteLogFile(fildes, m)==-1){
	  /**Error handle**/
	}

    }
    lt_ArrayLength++;
  }/*while 1==1*/

  fclose(tablesDeJour);

  if (UsMaRc==TRUE){
    iss=(isbd_table_str *)calloc(1,sizeof(isbd_table_str) );
    readISBDTable(iss->isbd_tbl);
  }

   return( 0 );
}

#else 
/*+---------------------------------------------------------------------+*/
/*! TABREAD()								!*/
/*! This version of the table readed is specifically for the ONE Tookit !*/
/*! It accepts a ':' or ',' or ' ' separated list of MARC format names  !*/
/*! and loads the table. 						!*/
/*! Retuns  0 for success or -1 for fail				!*/
/*+---------------------------------------------------------------------+*/
int TABREAD( char *ptr_format_list )
   {
   int i = 0;
   char *ffname;
   char *ttname;
   char *ptr_string;
   char error_message[200];
   char list[200];

   Boolean UsMaRc=FALSE;

   /*Initializing lt[] GLOBAL*/
   for (i=0;i<MAX_TABLES;i++)
      lt[i]=NULL;

   if ( !ptr_format_list || strlen( ptr_format_list ) > sizeof list )
      return( -1 );

   strcpy( list, ptr_format_list );
   ptr_format_list = list;

   i = 0;
   while( i < MAX_TABLES )
      {

      ptr_string = mystrtok( ptr_format_list, ":, " );
      ptr_format_list = NULL;

      if ( !ptr_string )
	 break;
      ffname = (char*)calloc( strlen( ptr_string ) + 1,  sizeof( char ));
      strcpy( ffname, ptr_string );
      
      ptr_string = mystrtok( NULL, ":, " );
      if ( !ptr_string )
	 break;

      ttname = (char*)calloc( strlen( ptr_string ) + 1,  sizeof( char ));
      strcpy( ttname, ptr_string );
      
printf("loading table from %s to %s\n", ffname, ttname);

      if (UsMaRc==FALSE)
          if (strcmp(ffname,"USMARC")==0 || strcmp(ttname,"USMARC")==0)
	UsMaRc=TRUE;

      lt[lt_ArrayLength]=readConversionSet(ffname,ttname);

      if (lt[lt_ArrayLength]==NULL)
         {
         sprintf(error_message, "TABREAD: unable to read conversion set " 
			      "for %s to %s\n", ffname, ttname);

         ONETK_WriteLogFile(fildes, error_message );
	 return( -1 );
         } /* End if */
      lt_ArrayLength++;
      i++;
      } /* End while */


   if (UsMaRc==TRUE)
      {
      iss=(isbd_table_str *)calloc(1,sizeof(isbd_table_str) );
      readISBDTable(iss->isbd_tbl);
      } /* End if */

   return( 0 );
}
#endif /* _ONETK_LIB_BUILD_ */

#endif /*MARC_MULTI_TABLE*/













