        /* * * * * * * * * * * * * * * * *     
	 *  MARCconv        Version 2.3  *
         *   File:        record_abs.c   *     
	 *      Record  Abstraction      *
	 *            functions          *
         *    Last update: 28/01/1997    *     
         *      Michael S. Preminger     *     
         *      BRODD, OSLO COLLEGE      *     
         *        THE ONE PROJECT        *     
         * * * * * * * * * * * * * * * * */    

/* * * * * * * * * * * * * * * * * * * * * * *
 * This file includes routines that access,  *
 * update  create and destroy fields and     *
 * subfields in a MARC record                *
 * * * * * * * * * * * * * * * * * * * * * * */


#include <setjmp.h>

#include "MARCproto.h"

extern jmp_buf marcconv_env;

subField *duplicateSubField(field *tofld, subField *fromSF){
  subField *toSF=createSubField(tofld,fromSF->subFieldCode);
  toSF->data=(char *)calloc((1+strlen(fromSF->data)),sizeof(char));
  strncpy( toSF->data, fromSF->data, strlen(fromSF->data));
  return toSF;
}

field *copyFieldOtherTag(record *rcd, 
			field *fromfld, 
			char *newTag)
{
 field *tofld;
 if (fromfld->datafield!=NULL){
   dataField *fromDtFld=fromfld->datafield, *toDtFld;
   int i;
   
   tofld=createField(rcd, newTag);
   toDtFld=tofld->datafield;
   toDtFld->indicators[0]=fromfld->datafield->indicators[0];
   toDtFld->indicators[1]=fromfld->datafield->indicators[1];
   toDtFld->indicators[2]=fromfld->datafield->indicators[2];
   
   for (i=0;i<fromDtFld->ArrayLength;i++){
     duplicateSubField(tofld,fromDtFld->subFields[i]);
   }
 }
 else{
   controlField *fromCtlFld=fromfld->ctrlfield, *toCtlFld;
   
   tofld=createField(rcd, newTag);
   toCtlFld=tofld->ctrlfield;
   toCtlFld->data=(char *)calloc((1+
			       strlen(fromCtlFld->data)),sizeof(char));
   strncpy(toCtlFld->data,fromCtlFld->data, 
	   strlen(fromfld->ctrlfield->data));
 }
 return tofld;


}

field *duplicateField(record *rcd, field *fromfld){
 field *tofld;
 if (fromfld->datafield!=NULL){
   dataField *fromDtFld=fromfld->datafield, *toDtFld;
   int i;
   
   tofld=createField(rcd, fromfld->datafield->tag);
   toDtFld=tofld->datafield;
   toDtFld->indicators[0]=fromfld->datafield->indicators[0];
   toDtFld->indicators[1]=fromfld->datafield->indicators[1];
   toDtFld->indicators[2]=fromfld->datafield->indicators[2];
   
   for (i=0;i<fromDtFld->ArrayLength;i++){
     duplicateSubField(tofld,fromDtFld->subFields[i]);
   }
 }
 else{
   controlField *fromCtlFld=fromfld->ctrlfield, *toCtlFld;
   
   tofld=createField(rcd, fromfld->ctrlfield->tag);
   toCtlFld=tofld->ctrlfield;
   toCtlFld->data=(char *)calloc((1+
			       strlen(fromCtlFld->data)),sizeof(char));
   strncpy(toCtlFld->data,fromCtlFld->data, 
	   strlen(fromfld->ctrlfield->data));
 }
 return tofld;
}

field *textToField(record *rcd, char *str){
  field *fld;
  dataField *df;
  subField *sf;
  int i;
  char *tmpdata=(char *)calloc(1000, sizeof(char));
  char indicators[3];
  char *tag=(char *)calloc(4, sizeof(char));
  char *tmpstr; /*Running pointer to str*/  
  tag=strncpy(tag,str,3);

    
  
  fld=createField(rcd,tag);
  
  df=fld->datafield;

  df->indicators[0]='%';
  df->indicators[1]=*(str+3);
  df->indicators[2]=*(str+4);
  
  tmpstr=str+6;
  sf=createSubField(fld, *tmpstr);
  tmpstr++;
  while(1==1){
    i=0;
    *tmpdata=*tmpstr;
    tmpstr++;
    while(*tmpstr!='$' && *tmpstr!='\0'){ 
      i++;
      *(tmpdata+i)=*tmpstr;
      tmpstr++;

    }
    sf->data=(char *)calloc(1+strlen(tmpdata), sizeof(char));
    strncpy(sf->data, tmpdata, strlen(tmpdata));
    memset(tmpdata,'\0',1000);
    if (*tmpstr=='\0') {
     free(tmpdata);
     free(tag);
     return fld;
    }
    tmpstr++;
    sf=createSubField(fld, *tmpstr);
    tmpstr++;
  }

  
  
  

  
  
}

void *removeField(record *rcd, int offset){
  int i;
  if (offset >= rcd->ArrayLength){
    return NULL;
  }
  /*printf("%x freeing line %d \n",  rcd->fields[offset], 155);*/
  freeField(rcd->fields[offset]);
  free(rcd->fields[offset]);
  /*We do it the "lengthy way" so that the 
   sequence of 999 fields is not changed*/
  for (i=offset+1; i< rcd->ArrayLength; i++){
    rcd->fields[i-1]=rcd->fields[i];
    rcd->fields[i]=NULL;
    strncpy(rcd->tags[i-1],rcd->tags[i],3);
    memset(rcd->tags[i],'\0',3);
  }
  rcd->ArrayLength--;
}

record *createRecord(){
  int i;
  record *tmpRcd=(record *)calloc(1, sizeof(record));

  /*Initialize format name*/
  memset(tmpRcd->format, '\0', 20);

  tmpRcd->ArrayLength=0;
  for (i=0;i<LEADER_LENGTH+1;i++)
    tmpRcd->leader[i]='\0';

  for (i=0;i<MAX_FIELDS_IN_RECORD;i++){
    /** to do
     * generalize tag_length away from 3
     **/
    memset(tmpRcd->tags[i],'\0',4);
    tmpRcd->lgt[i]=0;
    tmpRcd->pos[i]=0;
    tmpRcd->fields[i]=NULL;
  }
  tmpRcd->dataLength=0;
  
  return tmpRcd;
}

field *findField(record *rcd, char *tag){
  /*May be used both on in and out record*/
  int i;
  field *fld;
  
  /*Count backward so that if two fields with the same
   tag are present, the recent one will be used.*/
  for (i=rcd->ArrayLength-1;i>=0;i--){
    if (memcmp(rcd->tags[i],tag,3)==0)
      return rcd->fields[i];
  }
  return (field *)NULL;  
}
field *createField(record *rcd, char *tag ){
  field *fld;
  int i;
  char *tmpTag=(char *)calloc(1+strlen(tag), sizeof(char));
  /*
   * Done to avoid copying overlapping strings
   */
  strcpy(tmpTag, tag);
  if (rcd->ArrayLength>MAX_FIELDS_IN_RECORD-1){
    char m[200];
    sprintf(m, 
	    "warning: ( createField()\n   record size being exhausted. Extend MAX_FIELDS_IN_RECORD  in datastruct.h\n"
	    );
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
  }
  
  fld =(field *) calloc(1,sizeof(field));
  /*printf("%x tag=%s allocating, called from line %d\n", fld, tag, 230 );*/
  if (fld==NULL){
        char m[200];
	sprintf(m, 
		"createField()\n   malloc error\n"
		);
	if (ONETK_WriteLogFile(fildes, m)==-1){
	  /**Error handle**/
	}
	/*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
      
  }

  if (atoi(tag)>=10)		/** Set an "if" for failure?**/
    { /*Data Field*/
      dataField *dtfld=(dataField *)calloc(1, sizeof(dataField)); 
      char *indic_in;
/*I4*/
      /*dtfld->tag = (char*)malloc (4*sizeof(char));*/
      memset (dtfld->tag, '\0', 4);
      strncpy(dtfld->tag,tmpTag,3);
      /*memset(dtfld->tag+3,'\0',1);*/
      /*I5*/
      
      /*Initialize indicators*/
      memcpy(dtfld->indicators,"%  ",3);
      memset(dtfld->indicators+3,'\0',1);
      dtfld->ArrayLength=0;
      for (i=0;i<MAX_SUBFIELDS_IN_DATA_FIELD;i++){
	/*Initialization*/
	dtfld->subFields[i]=(subField *)NULL;
	dtfld->subFieldCodes[i]='\0';
      }
      
      fld->datafield=dtfld;
    }
  else
    {
      fld->ctrlfield=(controlField *)calloc(1, sizeof(controlField));
      strcpy(fld->ctrlfield->tag,tmpTag);
    }

  rcd->fields[rcd->ArrayLength]=fld;
  strncpy(rcd->tags[rcd->ArrayLength],tmpTag,3);
  rcd->ArrayLength++;
  if (tmpTag!=NULL)
    free(tmpTag);
 
  return fld;
}

subField *createSubField(field *fld, char code){
  dataField *dtfld;
  subField *sbFld;
  int i;
  
  if (fld->datafield==NULL){
      char m[200];
      sprintf(m, 
	      "createSubField(%s, %c)\n   Field sent with a NULL valued datafield\n",
	      "field", code
	      );
      if (ONETK_WriteLogFile(fildes, m)==-1){
	/**Error handle**/
	
      }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
     
  }

  dtfld=fld->datafield;

  if (dtfld->ArrayLength>=MAX_SUBFIELDS_IN_DATA_FIELD)
    {
      char m[200];
      sprintf(m, 
	      "createSubField()\n   Not enough positions in field-array.\n   Extend MAX_SUBFIELDS_IN_DATA_FIELD\n"
	      );
      if (ONETK_WriteLogFile(fildes, m)==-1){
	/**Error handle**/
	
      }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
    }
  
  sbFld=(subField *)calloc(1,sizeof(subField));
  if (sbFld==NULL){
    return (subField *) NULL;
  }
  dtfld->subFieldCodes[dtfld->ArrayLength]=code;
  dtfld->subFields[dtfld->ArrayLength]=sbFld;
  sbFld->subFieldCode=code;
  sbFld->index=dtfld->ArrayLength;
  dtfld->ArrayLength++;
  return sbFld;
}

field *accessField(record *rcd, 
		   char *tag)
     /**Check if it is at all necessary to 
       call it with the input record**/
{
  int i, correct_position;
  field *fld;
  if (atoi(tag)>999) return (field *)NULL;
  fld=findField(
		    rcd, 
		    tag);

  if (fld==(field  *)NULL)		/*Not  been built yet*/
    {
      /*CREATE FIELD AND PLACE IN RECORD*/
      fld=createField(rcd, tag);

    }
  
  return fld;
}

subField *findNextSubField(field *fp, 
			   int currentOffset,
			   char code)
{
  dataField *dfp=fp->datafield;
  subField *sfd;
  int i;
  for (i=currentOffset; i< dfp->ArrayLength; i++){
    if (code==dfp->subFieldCodes[i]){
      return dfp->subFields[i];
    }
  }
  return (subField *)NULL;
}

subField *findSubField(field *fp, 
		       char code)
{
  dataField *dfp=fp->datafield;
  subField *sfd;
  int i;
  for (i=0; i< dfp->ArrayLength; i++){
    if (code==dfp->subFieldCodes[i]){
      return dfp->subFields[i];
    }
  }
  return (subField *)NULL;
}

subField *accessSubField(field *fld, 
			 char sfcode)
{
  subField *sbfld;
  dataField *dtfld;
  int i, correct_position;
  /*Traverse subfields in fld to find the 
    subField*/
  
  sbfld=findSubField(fld,sfcode);
  if (sbfld==(subField *)NULL)		/*Not  been built yet*/
    {
      /*CREATE SUBFIELD AND PUT INTO A DATA FIELD*/

      sbfld=createSubField(fld, sfcode);
      if (sbfld==(subField *)NULL)
	{
	  char m[200];
	  sprintf(m, 
		  "accessSubField()\n   createSubField failed\n"
		  );
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	    
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	}
      else return sbfld;
    }
  else {
    return sbfld;
  }
}

field 
*ControlField_CopyDirect(record *rcd,
			 subPortionTable_Control *spPtr, 
			    int index,
			    char *string)
     /*Copies a string from the input field into the
       output field*/
{ 
  field *fld;
  char **tmpstr;		/*A pointer-pointer attached to the 
				  string to be updated*/
  char pref;
  char suff;
  int tmppos, tmplen,curlen; 

  /* * * * * * * * * 
   *   access      * 
   *   field       * 
   * * * * * * * * */
  fld=accessField(rcd, spPtr->toTag[index]); /*Creates if not yet existing*/ 
  if (fld==(field *)NULL  )
  {
    char m[200];
    sprintf(m, 
	    "ControlField_CopyDirect()\n  accessField failed\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
  }

  if (atoi(spPtr->toTag[index])<10)

    /*Data goes to a position (Known length subField) of a Control Field*/
    tmpstr=&fld->ctrlfield->data;
    
  else
    /*Data goes to a Known length subfield of a coded Data Field*/
    {
      subField *sbfld;

      /*Creates subfield if not yet existing, and attach tmpstr*/

      sbfld=accessSubField(fld, spPtr->toSubField[index]);
      if (sbfld==(subField *)NULL)
	{
	  char m[200];
	  sprintf(m, 
		  "ControlField_CopyDirect()\n  failed calling accessSubField\n");
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	}
      tmpstr=&sbfld->data;
    }
  if (*tmpstr==NULL)
    {
      /*string not previously accessed - build! */
      *tmpstr=(char *)calloc((1+LONGEST_SUB_PORTION),sizeof(char));
    }

  /*find Position in to-subField*/
  tmppos=spPtr->toPosition[index];
    tmplen=spPtr->toLength[index];
  /*
   *  Check if the string is long enough, reaching the position
   *  and the length. If not -- adjust.
   */
  curlen=strlen(*tmpstr);
  if (curlen<tmppos+tmplen){
    *tmpstr=realloc(*tmpstr, 
		    (curlen
		     +tmppos
		     +tmplen
		     +1)*sizeof(char)
		    );
    memset(*tmpstr+curlen, '\0', tmppos+tmplen-curlen +1 );
    memset(*tmpstr+curlen, ' ', 
	   tmppos+tmplen-curlen);
		
  }
 
  if (memcmp(spPtr->fromValue[index],"N",1)==0)

    /*modify the string if necessary*/
    string=Unimarc_date_modify(string,
			       spPtr,
			       index
			       );
			    
  /* * * * * * * * * * * 
   * place information * 
   *    in string      * 
   * * * * * * * * * * */
  if (tmppos+strlen(string)>LONGEST_SUB_PORTION){
    char m[200];
    sprintf(m, 
	    "ControlField_CopyDirect()\n  either string %s to long\n  or LONGEST_SUB_PORTION too short.\n need at least %d positions \n", string, LONGEST_SUB_PORTION);
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
     * exit(1);
     */
    longjmp(marcconv_env, 1);
    
  }
  

  strncpy(
	  /*Pointer arithmetic to find 
	    the exact position*/
	  *tmpstr+tmppos,
	  
	  string,

	  /*The length of "string", without
	    its terminating zero*/
	  strlen(string)

	  );
  
  return fld;

  }   
field *ControlField_CopyByPosition(record *rcd,
				   subPortionTable_Control *spPtr,
				   int relativePosition,
				   int index,
				   char *string)
     /*Copies a string from the input field into the
       output field*/
{ 
  field *fld;
  char **tmpstr;		/*A pointer-pointer attached to the 
				  string to be updated*/
  char pref;
  char suff;
  int tmppos, tmplen,curlen;

  /* * * * * * * * * 
   *   access      * 
   *   field       * 
   * * * * * * * * */
  fld=accessField(rcd, spPtr->toTag[index]); /*Creates if not yet existing*/ 
  if (fld==(field *)NULL  )
  {
    char m[200];
    sprintf(m, 
	    "ControlField_CopyDirect()\n   accessField failed\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
  }

  if (atoi(spPtr->toTag[index])<10)

    /*Data goes to a position (Known length subField) of a Control Field*/
    tmpstr=&fld->ctrlfield->data;
    
  else
    /*Data goes to a Known length subfield of a coded Data Field*/
    {
      subField *sbfld;

      /*Creates subfield if not yet existing, and attach tmpstr*/

      sbfld=accessSubField(fld, spPtr->toSubField[index]);
      if (sbfld==(subField *)NULL)
	{
	  char m[200];
	  sprintf(m, 
		  "ControlField_CopyDirect()\n   failed calling accessSubField\n");
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	}
      tmpstr=&sbfld->data;
    }
  if (*tmpstr==NULL)
    {
      /*string not previously accessed - build! */
      *tmpstr=(char *)calloc(LONGEST_SUB_PORTION+1,sizeof(char));
	
    }

  /*find Position in to-subField*/
  tmppos=spPtr->toPosition[index]+relativePosition;
  tmplen=spPtr->toLength[index];
  /*
   *  Check if the string is long enough, reaching the position
   *  and the length. If not -- adjust.
   */
  curlen=strlen(*tmpstr);
  if (curlen<tmppos+tmplen){
    *tmpstr=realloc(*tmpstr, 
		    (curlen
		     +tmppos
		     +tmplen
		     +1)*sizeof(char)
		    );
    memset(*tmpstr+curlen, '\0', tmppos+tmplen-curlen +1 );
    memset(*tmpstr+curlen, ' ', 
	   tmppos+tmplen-curlen);
		
  }
  
  if (memcmp(spPtr->fromValue[index],"N",1)==0)

    /*modify the string if necessary*/
    string=Unimarc_date_modify(string,
			       spPtr,
			       index
			       );
			    
  /* * * * * * * * * * * 
   * place information * 
   *    in string      * 
   * * * * * * * * * * */
  strncpy(
	  /*Pointer arithmetic to find 
	    the exact position*/
	  *tmpstr+tmppos,
	  
	  string,

	  /*The length of "string", without
	    its terminating zero*/
	  strlen(string)

	  );
  
  return fld;

}    
field *ControlField_CopySeqChar(record *rcd,
				 subPortionTable_Control *spPtr,
				   int index
				   ) /* Normally a single 
						    character*/
     /*Copies a characterof a sequence from the table into the
       output field*/
{ 
  field *fld;
  char **tmpstr;		/*A pointer-pointer attached to the 
				  string to be updated*/
  char pref;
  char suff;
  int tmppos, tmplen,curlen;
  int i;


  /* * * * * * * * * 
   *   access      * 
   *   field       * 
   * * * * * * * * */
  fld=accessField(rcd, spPtr->toTag[index]); /*Creates if not yet existing*/ 
  if (fld==(field *)NULL  )
  {
    char m[200];
    sprintf(m, 
	    "ControlField_CopySequence()\n  accessField failed\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
	 * exit(1);
	 */
	longjmp(marcconv_env, 1);
  }
  /*
   * Find first if there is a 
   * double occurence of a
   * character in the sequence (table error)
   */

  if (index>0)
    if (spPtr->fromPosition[index] == spPtr->fromPosition[index-1] )
      if (*spPtr->fromValue[index] == *spPtr->fromValue[index-1] )
	{
	  char m[200];
	  sprintf(m, 
		  "ControlField_CopySequence()\n   subsequent equal 'from values' in same position.\n to Tag: %s, Position: %d, Value: %s. Probably error in the control table\n",
		  spPtr->toTag[index], 
		  spPtr->fromPosition[index],
		  spPtr->fromValue[index]);
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  return fld;
	}
	
  if (atoi(spPtr->toTag[index])<10)

    /*Data goes to a position (Known length subField) of a Control Field*/
    tmpstr=&fld->ctrlfield->data;
    
  else
    /*Data goes to a Known length subfield of a coded Data Field*/
    {
      subField *sbfld;

      /*Creates subfield if not yet existing, and attach tmpstr*/

      sbfld=accessSubField(fld, spPtr->toSubField[index]);
      if (sbfld==(subField *)NULL)
	{
	  char m[200];
	  sprintf(m, 
		  "ControlField_CopySequence()\n   failed calling accessSubField\n");
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	}
      tmpstr=&sbfld->data;
    }
  if (*tmpstr==NULL)
    {
      /*string not previously accessed - build! */
      *tmpstr=(char *)calloc(LONGEST_SUB_PORTION+1,sizeof(char));
	
    }

  /*find Position in to-subField
   This is the first blank position
   within the character sequence.
   This is because the sequence should


   be left justified when shorter than
   toLength characters. We trust that
   sorting is superfluous, i.e character
   meanings in USMARC formats are the same,
   or at least same sorting order...*/
  /**Put characters in sorted!!**/

 
  for (i=0;i<spPtr->toLength[index];i++){
    if ( *(*tmpstr+spPtr->toPosition[index]+i)==' ' )
      break;
  }
  if (i<spPtr->toLength[index])
    tmppos=spPtr->toPosition[index]+i;
  else{
    char m[200];
    sprintf(m, 
	    "ControlField_CopySequence()\n   converting of a character sequence failed\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    return NULL;
  }
  tmplen=1;
  /*
   *  Check if the string is long enough, reaching the position
   *  and the length. If not -- adjust.
   */
  curlen=strlen(*tmpstr);
  if (curlen<tmppos+tmplen){
    *tmpstr=realloc(*tmpstr, 
		    (curlen
		     +tmppos
		     +tmplen
		     +1)*sizeof(char)
		    );
    memset(*tmpstr+curlen, '\0', tmppos+tmplen-curlen +1 );
    memset(*tmpstr+curlen, ' ', 
	   tmppos+tmplen-curlen);
		
  }
  
			    
  /* * * * * * * * * * * 
   * place information * 
   *    in string      * 
   * * * * * * * * * * */
  memset(
	  /*Pointer arithmetic to find 
	    the exact position*/
	 *tmpstr+tmppos,
	 
	 *spPtr->toValue[index],
	 
	 /*The length of "string", without
	   its terminating zero*/
	 tmplen

	  );
  
  return fld;

}    

field *ControlField_CopyFromTable(record *rcd,
				 subPortionTable_Control *spPtr, 
			      int index)
{ 
  field *fld;
  char **tmpstr;		/*A pointer-pointer attached to the 
				  string to be updated*/
  char pref;
  char suff;
  int tmppos, tmplen,curlen;
  subField *sbfld;

  /* * * * * * * * * 
   *   access      * 
   *   field       * 
   * * * * * * * * */
  fld=accessField(rcd, spPtr->toTag[index]); /*Creates if not yet existing*/ 
  if (fld==(field *)NULL  )
  {
    char m[200];
    sprintf(m, 
	    "ControlField_CopyFromTable()\n   accessField failed\n");
    if (ONETK_WriteLogFile(fildes, m)==-1){
      /**Error handle**/
    }
    /*
	 * exit(1);
	 */
    return NULL;
  }
  
  if (atoi(spPtr->toTag[index])<10)

    /*Data goes to a position (Known length subField) of a Control Field*/
    tmpstr=&fld->ctrlfield->data;
    
  else
    /*Data goes to a Known length subfield of a coded Data Field*/
    {


      /*Creates subfield if not yet existing, and attach tmpstr*/

      sbfld=accessSubField(fld, spPtr->toSubField[index]);
      if (sbfld==NULL) 
	{
	  char m[200];
	  sprintf(m, 
		  "ControlField_CopyFromTable()\n   failed calling accessSubField \n");
	  if (ONETK_WriteLogFile(fildes, m)==-1){
	    /**Error handle**/
	  }
	  /*
	   * exit(1);
	   */
	  longjmp(marcconv_env, 1);
	}
      tmpstr=&sbfld->data;
    }
  if (*tmpstr==NULL)
    {
      /*string not previously accessed - build! */
      *tmpstr=(char *)calloc(LONGEST_SUB_PORTION+1, sizeof(char));
	
    }

  /*find Position in to-subField*/
  tmppos=spPtr->toPosition[index];
    tmplen=spPtr->toLength[index];
  /*
   *  Check if the string is long enough, reaching the position
   *  and the length. If not -- adjust.
   */
  curlen=strlen(*tmpstr);
  if (curlen<tmppos+tmplen){
    *tmpstr=realloc(*tmpstr, 
		    (curlen
		     +tmppos
		     +tmplen
		     +1)*sizeof(char)
		    );
    memset(*tmpstr+curlen, '\0', tmppos+tmplen-curlen +1 );
    memset(*tmpstr+curlen, ' ', 
	   tmppos+tmplen-curlen);
		
  }
  
  /* * * * * * * * * * * 
   * place information * 
   *    in string      * 
   * * * * * * * * * * */
  strncpy(
	  /*Pointer arithmetic to find 
	    the exact "to" position*/
	  *tmpstr+tmppos,
						  
	  spPtr->toValue[index],
	  
	  /*The length of "string", without
	    its terminating zero*/
	  strlen(spPtr->toValue[index])

	  );
  return  fld;
}    

void *freeField(field *fld){
  int j;
  dataField *df;
  controlField *cf;
  if (fld->datafield!=NULL){
    df=fld->datafield;
    /*    if (df->tag!=NULL) 
	  free (df->tag);*/
    /*printf("%x tag=%s freeing line %d \n",  fld, df->tag, 953);*/
    for (j=0;j<df->ArrayLength;j++){
      if (df->subFields[j]!=NULL){
	if (df->subFields[j]->data!=NULL){
	  /*printf("%x freeing line %d \n",  df->subFields[j]->data, 955);*/
	  free(df->subFields[j]->data);
	  df->subFields[j]->data=NULL;
	}
	free(df->subFields[j]);
	df->subFields[j]=NULL;
      }
    }
    if (df->text!=NULL){
      free(df->text);
      df->text=NULL;
    }
    free(df);
    df=NULL;
  }
  if (fld->ctrlfield!=NULL){
    free(fld->ctrlfield->data);
    free(fld->ctrlfield);
  }


  /**Coded data field**/

}
void *freeRecord(record *rcd)
{
  int i;
  for (i=0;i<rcd->ArrayLength;i++){
    if (rcd->fields[i]!=NULL)
      {
		printf("%x calling free, 984\n", rcd->fields[i]);
	freeField(rcd->fields[i]);
	free(rcd->fields[i]);
      }
  }
  free(rcd);
}
void *freeInRecord(record *rcd)
{
  int i;
  for (i=0;i<rcd->ArrayLength;i++){
    if (rcd->fields[i]!=NULL)
      {
	/*printf("%x calling free, 997\n", rcd->fields[i]);*/
	freeField(rcd->fields[i]);
	free(rcd->fields[i]);
      }
  }
  free(rcd);
}
void *freeOutRecord(record *rcd)
{/**/
  int i;
  for (i=0;i<rcd->ArrayLength;i++){
    if (rcd->fields[i]!=NULL 
	/*&& memcmp(rcd->tags[i],"999",3)!=0*/)
      {
	/*	printf("%x, calling free 1011\n", rcd->fields[i]);*/
	freeField(rcd->fields[i]);
	free(rcd->fields[i]);
      }
  }
  free(rcd);
}
