/*  Module Name: path.c
    Version: $Revision$
    Author: Barry Perryman

    Copyright 1996, OPAC Network in Europe project partners.

    Date of Issue:

    QA Acceptance:
    Date of Acceptance:

    Description:
    Provide support for filename location in either a user define path,
    as specified in the TOOLBOXPATH environment variable, or the default
    path.

*/

#include <sys/param.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <onetk.h>

#ifndef DEFAULTPATH
#define DEFAULTPATH "/usr/local/lib/onetk:/usr/share/onetk:."
#endif /* not DEFAULTPATH */

static void ONETK_ComposeFileName(char *retName, char *path, char *fileName);
static char *ONETK_RetrieveSearchPath(void);



/*
 * ONETK_FilePath: Search through either a user defined search path or the
 * default search path for an occurance of the supplied filename. If a match is
 * found that is readable, then return a pointer to the files location. If it
 * isn't found return NULL
 */
char *
ONETK_FilePath(char *fname)
{
    char *rfname = malloc(MAXPATHLEN+1);
    char *srchPath, *tmpPath;
    char found = 0;

	if (*fname == '/')
	{
		/*
		 * As the file starts with a leading '/' we have an absolute path
		 * name for the file, and as such we shouldn't try and process
		 * this.  Copy this path into our return file name so that the
		 * calling routine is still able to free the memory as usual.
		 */
		strcpy(rfname, fname);
	}
	else
	{
		srchPath = ONETK_RetrieveSearchPath();
		tmpPath = strtok(srchPath, ":");
		while (tmpPath != NULL)
		{
			ONETK_ComposeFileName(rfname, tmpPath, fname);
			if (access(rfname, R_OK) == 0)
			{
				found = 1;
				break;
			}
			tmpPath = strtok(NULL, ":");
		}
		if (!found)
		{
			free(rfname);
			rfname = NULL;
		}
		free(srchPath);
	}
	return rfname;
}

/*
 * ONETK_RetrieveSearchPath: This will retrieve from the environment
 * variable TOOLBOXPATH the users desired search path. If this seach path
 * starts or finishes with a ':' then the default search path will be
 * pre/appended. If the environment variable does not exist then the
 * default search path is returned.  The pointer return by this function
 * is to malloc'd memory, and should be free'd by the calling routine when
 * it has finished with the data.
 */
static char *
ONETK_RetrieveSearchPath(void)
{
	char *tmp1, *tmp2;
	int len, defLen = strlen(DEFAULTPATH);

	if ((tmp1 = getenv("TOOLBOXPATH")) == NULL)
	{
		/* user doesn't have a TOOLBOXPATH set */
		tmp2 = malloc(defLen+1);
		strcpy(tmp2, DEFAULTPATH);
	}
	else
	{
		len = strlen(tmp1);
		if (tmp1[0] == ':')
		{
			/* pre-pend the default search path */
			tmp2 = malloc(defLen+len+2);	/* seperator ':' and '\0' */
			strcpy(tmp2, DEFAULTPATH);
			strcat(tmp2, ":");
			strcat(tmp2, tmp1);
		}
		else if (tmp1[len-1] == ':')
		{
			/* append the default search path */
			tmp2 = malloc(defLen+len+2);
			strcpy(tmp2, tmp1);
			strcat(tmp2, ":");
			strcat(tmp2, DEFAULTPATH);
		}
		else
		{
			tmp2 = malloc(len+1);
			strcpy(tmp2, tmp1);
		}
	}
	return tmp2;
}

/*
 * ONETK_ComposeFileName: this takes a pointer to a char, in which the
 * file name and path are entered into; a path entry and a filename. If
 * the path entry does not end in a '/' then one is appended to the path
 * before the filename is added on.
 */
static void
ONETK_ComposeFileName(char *retName, char *path, char *fileName)
{
	int len = strlen(path);

	strcpy(retName, path);
	if (path[len-1] != '/')
	{
		strcat(retName, "/");
	}
	strcat(retName, fileName);
}
int main(){
     char *test;
     test=ONETK_FilePath("fromToFormat.txt");
  
}
